ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     ng2d.f -- Subroutines for drawing contor lines and vector
c               diagrams of 2D field data by using Ngraph
c 
c              Ver.0.8     Tue May 16 23:29:40 JST 2000
c
c             Copyright Amane Tanaka (amane@bd5.so-net.ne.jp)
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
      subroutine ngcc1a(nd,fname,nx,ny,x,y,z,nc)
c     *****************************************************************
c     *  ngcc1a - Contour lines with Colors No.1, All area
c     *   ( zc(n) = zmin+(n-1)*dz, n=1,...,nc, zc(nc)=zmax )
c     *                  
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real          z(nx,ny) : values of grid points
c     *   integer             nc : number of contour levels
c     *    
c     *****************************************************************
      dimension x(nx,ny),y(nx,ny),z(nx,ny)
      character*80 fname 
c
      call ngrnga(nx,ny,z,zmin,imin,jmin,zmax,imax,jmax)
      dz = (zmax-zmin)/float(nc-1)
c
      call ngcc3a(nd,fname,nx,ny,x,y,z,nc,zmin,dz)
c
      return
      end
c
c
c
      subroutine ngcc2a(nd,fname,nx,ny,x,y,z,nc)
c     *****************************************************************
c     *  ngcc2a - Contour lines with Colors, No.2, All area
c     *   ( zc(n) = zc(n+1)*ratio, zc(nc)=zmax, zc(1)=zmin )  
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real          z(nx,ny) : values of grid points
c     *   integer             nc : number of contour levels
c     *    
c     *****************************************************************
      dimension x(nx,ny),y(nx,ny),z(nx,ny)
      character*80 fname 
c
      call ngrnga(nx,ny,z,zmin,imin,jmin,zmax,imax,jmax) 
      if (zmin.le.0.0) then
         ratio = 0.1
      else
         a = (log(zmin)-log(zmax))/(float(nc)-1.0)
         ratio = exp(a)
      endif
c
      call ngcc4a(nd,fname,nx,ny,x,y,z,nc,zmax,ratio)
c
      return
      end
c
c
c
      subroutine ngcc3a(nd,fname,nx,ny,x,y,z,nc,zmin,dz)
c     *****************************************************************
c     *  ngcc3a - Contour lines with Colors, No.3, All area
c     *   ( zc(n) = zmin+(n-1)*dz, n=1,...,nc )
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real          z(nx,ny) : values of grid points
c     *   integer             nc : number of contour levels
c     *   real              zmin : minimum value of contour levels
c     *   real                dz : interval between contour levels 
c     *    
c     *****************************************************************
      include 'ng2d_par.f'
      dimension x(nx,ny),y(nx,ny),z(nx,ny),zc(ncmax)
      character*80 fname 
c
      do 10 i=1,nc
         zc(i)=zmin+dz*(i-1)
 10   continue
c
      call ngcc5a(nd,fname,nx,ny,x,y,z,nc,zc)
c
      return
      end
c
c
c
      subroutine ngcc4a(nd,fname,nx,ny,x,y,z,nc,zmax,ratio)
c     *****************************************************************
c     *  ngcc4a - Contour lines with Colors, No.4, All area
c     *   ( zc(n) = zc(n+1)*ratio, zc(nc)=zmax )
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real          z(nx,ny) : values of grid points
c     *   integer             nc : number of contour levels
c     *   real              zmax : max value of contour levels
c     *   real             ratio : ratio of each contour levels 
c     *    
c     *****************************************************************
      include 'ng2d_par.f'
      dimension x(nx,ny),y(nx,ny),z(nx,ny),zc(ncmax)
      character*80 fname 
c
      zc(nc)=zmax
c
      do 10 i=1,nc-1
         zc(nc-i)=zc(nc-i+1)*ratio
 10   continue
c
      call ngcc5a(nd,fname,nx,ny,x,y,z,nc,zc)
c
      return
      end
c
c
c
      subroutine ngcc5a(nd,fname,nx,ny,x,y,z,nc,zc)
c     *****************************************************************
c     *  ngcc5a - Contour lines with Colors, No.5, All area
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real          z(nx,ny) : values of grid points
c     *   integer             nc : number of contour levels
c     *   real            zc(nc) : array of contour levels
c     *    
c     *****************************************************************
      include 'ng2d_par.f'
      dimension x(nx,ny),y(nx,ny),z(nx,ny),zc(*)
      character*80 fname,dname,nstr 
      integer red(ncmax),green(ncmax),blue(ncmax)
c
      call ngclm(nc,red,green,blue)
c
      dname = fname(1:nglen(fname)) // '.part' 
      open(nd,file=dname,status='UNKNOWN')
      do 30 k=1,nc
         call ngsti(k,nstr)
         dname = fname(1:nglen(fname)) // '.' // nstr(1:nglen(nstr))
         iy = 1000+(nc-k)*300
         call ngslin(nd,dname,zc(k),red(k),green(k),blue(k),1,17500,iy)
 30   continue
      close(nd)
c
      do 10 k=1,nc
         call ngsti(k,nstr)
         dname = fname(1:nglen(fname)) // '.' // nstr(1:nglen(nstr))
         open(nd,file=dname,status='UNKNOWN')
         call ngc0a(nd,nx,ny,x,y,z,zc(k))
         close(nd)
 10   continue
c
      return
      end
c
c
c
      subroutine ngcc1s(nd,fname,nx,ny,x,y,z,nc,i1,j1,i2,j2,nmsk)
c     *****************************************************************
c     *  ngcc1s - Contour lines with Colors No.1, Specified area
c     *   ( zc(n) = zmin+(n-1)*dz, n=1,...,nc, zc(nc)=zmax )
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real          z(nx,ny) : values of grid points
c     *   integer             nc : number of contour levels
c     *   integer    i1,i2,j1,j2 : range of drowing area by grid index
c     *   integer    nmsk(nx,ny) : mask array, 0:draw, else:neglect     
c     *    
c     *****************************************************************
      dimension x(nx,ny),y(nx,ny),z(nx,ny)
      integer nmsk(nx,ny)
      character*80 fname 
c
      call ngrngs(nx,ny,z,zmin,imin,jmin,zmax,imax,jmax,
     &     i1,j1,i2,j2,nmsk) 
      dz = (zmax-zmin)/float(nc-1)
c
      call ngcc3s(nd,fname,nx,ny,x,y,z,nc,zmin,dz,i1,j1,i2,j2,nmsk)
c
      return
      end
c
c
c
      subroutine ngcc2s(nd,fname,nx,ny,x,y,z,nc,i1,j1,i2,j2,nmsk)
c     *****************************************************************
c     *  ngcc2s - Contour lines with Colors, No.2, Specified area
c     *   ( zc(n) = zc(n+1)*ratio, zc(nc)=zmax, zc(1)=zmin )  
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real          z(nx,ny) : values of grid points
c     *   integer             nc : number of contour levels
c     *   integer    i1,i2,j1,j2 : range of drowing by grid index
c     *   integer    nmsk(nx,ny) : mask array, 0:draw, else:neglect     
c     *    
c     *****************************************************************
      dimension x(nx,ny),y(nx,ny),z(nx,ny)
      integer nmsk(nx,ny)
      character*80 fname 
c
      call ngrngs(nx,ny,z,zmin,imin,jmin,zmax,imax,jmax,
     &     i1,j1,i2,j2,nmsk)
      if (zmin.le.0.0) then
         ratio = 0.1
      else
         a = (log(zmin)-log(zmax))/(float(nc)-1.0)
         ratio = exp(a)
      endif
c
      call ngcc4s(nd,fname,nx,ny,x,y,z,nc,zmax,ratio,i1,j1,i2,j2,nmsk)
c
      return
      end
c
c
c
      subroutine ngcc3s(nd,fname,nx,ny,x,y,z,nc,zmin,dz,
     &     i1,j1,i2,j2,nmsk)
c     *****************************************************************
c     *  ngcc3s - Contour lines with Colors, No.3, Specified area
c     *   ( zc(n) = zmin+(n-1)*dz, n=1,...,nc )
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real          z(nx,ny) : values of grid points
c     *   integer             nc : number of contour levels
c     *   real              zmin : minimum value of contour levels
c     *   real                dz : interval between contour levels 
c     *   integer    i1,j1,i2,j2 : range of drowing by grid index
c     *   integer    nmsk(nx,ny) : mask array, 0:draw, else:neglect     
c     *    
c     *****************************************************************
      include 'ng2d_par.f'
      dimension x(nx,ny),y(nx,ny),z(nx,ny),zc(ncmax)
      integer nmsk(nx,ny)
      character*80 fname 
c
      do 10 i=1,nc
         zc(i)=zmin+dz*(i-1)
 10   continue
c
      call ngcc5s(nd,fname,nx,ny,x,y,z,nc,zc,i1,j1,i2,j2,nmsk)
c
      return
      end
c
c
c
      subroutine ngcc4s(nd,fname,nx,ny,x,y,z,nc,zmax,ratio,
     &     i1,j1,i2,j2,nmsk)
c     *****************************************************************
c     *  ngcc4s - Contour lines with Colors, No.4, Specified area
c     *   ( zc(n) = zc(n+1)*ratio, zc(nc)=zmax )
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real          z(nx,ny) : values of grid points
c     *   integer             nc : number of contour levels
c     *   real              zmax : max value of contour levels
c     *   real             ratio : ratio of each contour levels 
c     *   integer    i1,j1,i2,j2 : range of drowing by grid index
c     *   integer    nmsk(nx,ny) : mask array, 0:draw, else:neglect     
c     *    
c     *****************************************************************
      include 'ng2d_par.f'
      dimension x(nx,ny),y(nx,ny),z(nx,ny),zc(ncmax)
      integer nmsk(nx,ny)
      character*80 fname 
c
      zc(nc)=zmax
c
      do 10 i=1,nc-1
         zc(nc-i)=zc(nc-i+1)*ratio
 10   continue
c
      call ngcc5s(nd,fname,nx,ny,x,y,z,nc,zc,i1,j1,i2,j2,nmsk)
c
      return
      end
c
c
c
      subroutine ngcc5s(nd,fname,nx,ny,x,y,z,nc,zc,i1,j1,i2,j2,nmsk)
c     *****************************************************************
c     *  ngcc5s - Contour lines with Colors, No.5, Specified area
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real          z(nx,ny) : values of grid points
c     *   integer             nc : number of contour levels
c     *   real            zc(nc) : array of contour levels
c     *   integer    i1,j1,i1,j2 : range of drowing by grid index
c     *   integer    nmsk(nx,ny) : mask array, 0:draw, else:f7neglect     
c     *    
c     *****************************************************************
      include 'ng2d_par.f'
      dimension x(nx,ny),y(nx,ny),z(nx,ny),zc(*)
      integer nmsk(nx,ny)
      character*80 fname,dname,nstr 
      integer red(ncmax),green(ncmax),blue(ncmax)
c
      call ngclm(nc,red,green,blue)
c
      dname = fname(1:nglen(fname)) // '.part' 
      open(nd,file=dname,status='UNKNOWN')
      do 30 k=1,nc
         call ngsti(k,nstr)
         dname = fname(1:nglen(fname)) // '.' // nstr(1:nglen(nstr))
         iy = 1000+(nc-k)*300
         call ngslin(nd,dname,zc(k),red(k),green(k),blue(k),1,17500,iy)
 30   continue
      close(nd)
c
      do 10 k=1,nc
         call ngsti(k,nstr)
         dname = fname(1:nglen(fname)) // '.' // nstr(1:nglen(nstr))
         open(nd,file=dname,status='UNKNOWN')
         call ngc0s(nd,nx,ny,x,y,z,i1,j1,i2,j2,nmsk,zc(k))
         close(nd)
 10   continue
c
      return
      end
c
c
c
      subroutine ngcm1a(nd,fname,nx,ny,x,y,z,nc)
c     *****************************************************************
c     *  ngcm1a - Contour lines without Colors No.1, All area
c     *   ( zc(n) = zmin+(n-1)*dz, n=1,...,nc, zc(nc)=zmax )
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real          z(nx,ny) : values of grid points
c     *   integer             nc : number of contour levels
c     *    
c     *****************************************************************
      dimension x(nx,ny),y(nx,ny),z(nx,ny)
      character*80 fname 
c
      call ngrnga(nx,ny,z,zmin,imin,jmin,zmax,imax,jmax) 
      dz = (zmax-zmin)/float(nc-1)
c
      call ngcm3a(nd,fname,nx,ny,x,y,z,nc,zmin,dz)
c
      return
      end
c
c
c
      subroutine ngcm2a(nd,fname,nx,ny,x,y,z,nc)
c     *****************************************************************
c     *  ngcm2a - Contour lines without Colors, No.2, All area
c     *   ( zc(n) = zc(n+1)*ratio, zc(nc)=zmax, zc(1)=zmin )  
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real          z(nx,ny) : values of grid points
c     *   integer             nc : number of contour levels
c     *    
c     *****************************************************************
      dimension x(nx,ny),y(nx,ny),z(nx,ny)
      character*80 fname 
c
      call ngrnga(nx,ny,z,zmin,imin,jmin,zmax,imax,jmax)
      if (zmin.le.0.0) then
         ratio = 0.1
      else
         a = (log(zmin)-log(zmax))/(float(nc)-1.0)
         ratio = exp(a)
      endif
c
      call ngcm4a(nd,fname,nx,ny,x,y,z,nc,zmax,ratio)
c
      return
      end
c
c
c
      subroutine ngcm3a(nd,fname,nx,ny,x,y,z,nc,zmin,dz)
c     *****************************************************************
c     *  ngcm3a - Contour lines without Colors, No.3, All area
c     *   ( zc(n) = zmin+(n-1)*dz, n=1,...,nc )
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real          z(nx,ny) : values of grid points
c     *   integer             nc : number of contour levels
c     *   real              zmin : minimum value of contour levels
c     *   real                dz : interval between contour levels 
c     *    
c     *****************************************************************
      include 'ng2d_par.f'
      dimension x(nx,ny),y(nx,ny),z(nx,ny),zc(ncmax)
      character*80 fname 
c
      do 10 i=1,nc
         zc(i)=zmin+dz*(i-1)
 10   continue
c
      call ngcm5a(nd,fname,nx,ny,x,y,z,nc,zc)
c
      return
      end
c
c
c
      subroutine ngcm4a(nd,fname,nx,ny,x,y,z,nc,zmax,ratio)
c     *****************************************************************
c     *  ngcm4a - Contour lines without Colors, No.4, All area
c     *   ( zc(n) = zc(n+1)*ratio, zc(nc)=zmax )
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real          z(nx,ny) : values of grid points
c     *   integer             nc : number of contour levels
c     *   real              zmax : max value of contour levels
c     *   real             ratio : ratio of each contour levels 
c     *    
c     *****************************************************************
      include 'ng2d_par.f'
      dimension x(nx,ny),y(nx,ny),z(nx,ny),zc(ncmax)
      character*80 fname 
c
      zc(nc)=zmax
c
      do 10 i=1,nc-1
         zc(nc-i)=zc(nc-i+1)*ratio
 10   continue
c
      call ngcm5a(nd,fname,nx,ny,x,y,z,nc,zc)
c
      return
      end
c
c
c
      subroutine ngcm5a(nd,fname,nx,ny,x,y,z,nc,zc)
c     *****************************************************************
c     *  ngcm5a - Contour lines without Colors, No.5, All area
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real          z(nx,ny) : values of grid points
c     *   integer             nc : number of contour levels
c     *   real            zc(nc) : array of contour levels
c     *    
c     *****************************************************************
      dimension x(nx,ny),y(nx,ny),z(nx,ny),zc(*)
      character*80 fname,dname
c
      dname = fname(1:nglen(fname)) // '.part' 
      open(nd,file=dname,status='UNKNOWN')
      dname = fname(1:nglen(fname)) // '.dat'
      call ngslin(nd,dname,0.0,0,0,0,0,1,1)
      close(nd)
c
      dname = fname(1:nglen(fname)) // '.dat'
      open(nd,file=dname,status='UNKNOWN')
      do 10 k=1,nc
         call ngc0a(nd,nx,ny,x,y,z,zc(k))
 10   continue
      close(nd)
c
      return
      end
c
c
      subroutine ngcm1s(nd,fname,nx,ny,x,y,z,nc,i1,j1,i2,j2,nmsk)
c     *****************************************************************
c     *  ngcm1s - Contour lines without Colors No.1, Specified area
c     *   ( zc(n) = zmin+(n-1)*dz, n=1,...,nc, zc(nc)=zmax )
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real          z(nx,ny) : values of grid points
c     *   integer             nc : number of contour levels
c     *   integer    i1,j1,i2,j2 : range of drowing by grid index
c     *   integer    nmsk(nx,ny) : mask array, 0:draw, else:neglect     
c     *    
c     *****************************************************************
      dimension x(nx,ny),y(nx,ny),z(nx,ny)
      integer nmsk(nx,ny)
      character*80 fname 
c
      call ngrngs(nx,ny,z,zmin,imin,jmin,zmax,imax,jmax,
     &     i1,j1,i2,j2,nmsk) 
      dz = (zmax-zmin)/float(nc-1)
c
      call ngcm3s(nd,fname,nx,ny,x,y,z,nc,zmin,dz,i1,j1,i2,j2,nmsk)
c
      return
      end
c
c
c
      subroutine ngcm2s(nd,fname,nx,ny,x,y,z,nc,i1,j1,i2,j2,nmsk)
c     *****************************************************************
c     *  ngcm2s - Contour lines without Colors, No.2, Specified area
c     *   ( zc(n) = zc(n+1)*ratio, zc(nc)=zmax, zc(1)=zmin )  
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real          z(nx,ny) : values of grid points
c     *   integer             nc : number of contour levels
c     *   integer    i1,j1,i2,j2 : range of drowing by grid index
c     *   integer    nmsk(nx,ny) : mask array, 0:draw, else:neglect     
c     *    
c     *****************************************************************
      dimension x(nx,ny),y(nx,ny),z(nx,ny)
      integer nmsk(nx,ny)
      character*80 fname 
c
      call ngrngs(nx,ny,z,zmin,imin,jmin,zmax,imax,jmax,
     &     i1,j1,i2,j2,nmsk)
      if (zmin.le.0.0) then
         ratio = 0.1
      else
         a = (log(zmin)-log(zmax))/(float(nc)-1.0)
         ratio = exp(a)
      endif
c
      call ngcm4s(nd,fname,nx,ny,x,y,z,nc,zmax,ratio,i1,j1,i2,j2,nmsk)
c
      return
      end
c
c
c
      subroutine ngcm3s(nd,fname,nx,ny,x,y,z,nc,zmin,dz,
     &     i1,j1,i2,j2,nmsk)
c     *****************************************************************
c     *  ngcm3s - Contour lines without Colors, No.3, Specified area
c     *   ( zc(n) = zmin+(n-1)*dz, n=1,...,nc )
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real          z(nx,ny) : values of grid points
c     *   integer             nc : number of contour levels
c     *   real              zmin : minimum value of contour levels
c     *   real                dz : interval between contour levels 
c     *   integer    i1,j1,i2,j2 : range of drowing by grid index
c     *   integer    nmsk(nx,ny) : mask array, 0:draw, else:neglect     
c     *    
c     *****************************************************************
      include 'ng2d_par.f'
      dimension x(nx,ny),y(nx,ny),z(nx,ny),zc(ncmax)
      integer nmsk(nx,ny)
      character*80 fname 
c
      do 10 i=1,nc
         zc(i)=zmin+dz*(i-1)
 10   continue
c
      call ngcm5s(nd,fname,nx,ny,x,y,z,nc,zc,i1,j1,i2,j2,nmsk)
c
      return
      end
c
c
c
      subroutine ngcm4s(nd,fname,nx,ny,x,y,z,nc,zmax,ratio,
     &     i1,j1,i2,j2,nmsk)
c     *****************************************************************
c     *  ngcm4s - Contour lines without Colors, No.4, Specified area
c     *   ( zc(n) = zc(n+1)*ratio, zc(nc)=zmax )
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real          z(nx,ny) : values of grid points
c     *   integer             nc : number of contour levels
c     *   real              zmax : max value of contour levels
c     *   real             ratio : ratio of each contour levels 
c     *   integer    i1,j1,i2,j2 : range of drowing by grid index
c     *   integer    nmsk(nx,ny) : mask array, 0:draw, else:neglect     
c     *    
c     *****************************************************************
      include 'ng2d_par.f'
      dimension x(nx,ny),y(nx,ny),z(nx,ny),zc(ncmax)
      integer nmsk(nx,ny)
      character*80 fname 
c
      zc(nc)=zmax
c
      do 10 i=1,nc-1
         zc(nc-i)=zc(nc-i+1)*ratio
 10   continue
c
      call ngcm5s(nd,fname,nx,ny,x,y,z,nc,zc,i1,j1,i2,j2,nmsk)
c
      return
      end
c
c
c
      subroutine ngcm5s(nd,fname,nx,ny,x,y,z,nc,zc,i1,j1,i2,j2,nmsk)
c     *****************************************************************
c     *  ngcm5s - Contour lines without Colors, No.5, Specified area
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real          z(nx,ny) : values of grid points
c     *   integer             nc : number of contour levels
c     *   real            zc(nc) : array of contour levels
c     *   integer    i1,j1,i2,j2 : range of drowing by grid index
c     *   integer    nmsk(nx,ny) : mask array, 0:draw, else:neglect     
c     *    
c     *****************************************************************
      dimension x(nx,ny),y(nx,ny),z(nx,ny),zc(*)
      integer nmsk(nx,ny)
      character*80 fname,dname,nstr 
c
      dname = fname(1:nglen(fname)) // '.part' 
      open(nd,file=dname,status='UNKNOWN')
      dname = fname(1:nglen(fname)) // '.dat'
      call ngslin(nd,dname,0.0,0,0,0,0,1,1)
      close(nd)
c
      dname = fname(1:nglen(fname)) // '.dat'
      open(nd,file=dname,status='UNKNOWN')
      do 10 k=1,nc
         call ngc0s(nd,nx,ny,x,y,z,i1,j1,i2,j2,nmsk,zc(k))
 10   continue
      close(nd)
c
      return
      end
c
c
c
      subroutine ngcf1a(nd,fname,nx,ny,x,y,z,nc,ires,jres)
c     *****************************************************************
c     *  ngcf1a - Contour Fill No.1, All area
c     *   ( zc(n) = zmin+(n-1)*dz, n=1,...,nc, zc(nc)=zmax )
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real          z(nx,ny) : values of grid points
c     *   integer             nc : number of contour levels
c     *   integer      ires,jres : resolutions in x,y-directions    
c     *    
c     *****************************************************************
      dimension x(nx,ny),y(nx,ny),z(nx,ny)
      character*80 fname 
c
      call ngrnga(nx,ny,z,zmin,imin,jmin,zmax,imax,jmax) 
      dz = (zmax-zmin)/float(nc-1)
c
      call ngcf3a(nd,fname,nx,ny,x,y,z,nc,zmin,dz,ires,jres)
c
      return
      end
c
c
c
      subroutine ngcf2a(nd,fname,nx,ny,x,y,z,nc,ires,jres)
c     *****************************************************************
c     *  ngcf2a - Contour Fill No.2, All area
c     *   ( zc(n) = zc(n+1)*ratio, zc(nc)=zmax, zc(1)=zmin )  
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real          z(nx,ny) : values of grid points
c     *   integer             nc : number of contour levels
c     *   integer      ires,jres : resolutions in x,y-directions    
c     *    
c     *****************************************************************
      dimension x(nx,ny),y(nx,ny),z(nx,ny)
      character*80 fname 
c
      call ngrnga(nx,ny,z,zmin,imin,jmin,zmax,imax,jmax)
      if (zmin.le.0.0) then
         ratio = 0.1
      else
         a = (log(zmin)-log(zmax))/(float(nc)-1.0)
         ratio = exp(a)
      endif
c
      call ngcf4a(nd,fname,nx,ny,x,y,z,nc,zmax,ratio,ires,jres)
c
      return
      end
c
c
c
      subroutine ngcf3a(nd,fname,nx,ny,x,y,z,nc,zmin,dz,ires,jres)
c     *****************************************************************
c     *  ngcf3a - Contour Fill No.3, All area
c     *   ( zc(n) = zmin+(n-1)*dz, n=1,...,nc )
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real          z(nx,ny) : values of grid points
c     *   integer             nc : number of contour levels
c     *   real              zmin : minimum value of contour levels
c     *   real                dz : interval between contour levels 
c     *   integer      ires,jres : resolutions in x,y-directions    
c     *    
c     *****************************************************************
      include 'ng2d_par.f'
      dimension x(nx,ny),y(nx,ny),z(nx,ny),zc(ncmax)
      character*80 fname 
c
      do 10 i=1,nc
         zc(i)=zmin+dz*(i-1)
 10   continue
c
      call ngcf5a(nd,fname,nx,ny,x,y,z,nc,zc,ires,jres)
c
      return
      end
c
c
c
      subroutine ngcf4a(nd,fname,nx,ny,x,y,z,nc,zmax,ratio,ires,jres)
c     *****************************************************************
c     *  ngcf4a - Contour Fill No.4, All area
c     *   ( zc(n) = zc(n+1)*ratio, zc(nc)=zmax )
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real          z(nx,ny) : values of grid points
c     *   integer             nc : number of contour levels
c     *   real              zmax : max value of contour levels
c     *   real             ratio : ratio of each contour levels 
c     *   integer      ires,jres : resolutions in x,y-directions    
c     *    
c     *****************************************************************
      include 'ng2d_par.f'
      dimension x(nx,ny),y(nx,ny),z(nx,ny),zc(ncmax)
      character*80 fname 
c
      zc(nc)=zmax
c
      do 10 i=1,nc-1
         zc(nc-i)=zc(nc-i+1)*ratio
 10   continue
c
      call ngcf5a(nd,fname,nx,ny,x,y,z,nc,zc,ires,jres)
c
      return
      end
c
c
c
      subroutine ngcf5a(nd,fname,nx,ny,x,y,z,nc,zc,ires,jres)
c     *****************************************************************
c     *  ngcf5a - Contour Fill No.5, All area
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real          z(nx,ny) : values of grid points
c     *   integer             nc : number of contour lines
c     *   real             zc(nc) : value of contour lines
c     *   integer      ires,jres : resolutions in x,y-directions    
c     *    
c     *****************************************************************
      include 'ng2d_par.f'
      dimension x(nx,ny),y(nx,ny),z(nx,ny),zc(*)
      character*80 fname,dname,nstr 
      integer red(ncmax),green(ncmax),blue(ncmax)
c
      call ngclm(nc-1,red,green,blue)
c
      dname = fname(1:nglen(fname)) // '.part' 
      open(nd,file=dname,status='UNKNOWN')
      do 30 k=1,nc-1
         call ngsti(k,nstr)
         dname = fname(1:nglen(fname)) // '.' // nstr(1:nglen(nstr))
         iy = 1000+(nc-k)*300
         call ngsrec(nd,dname,zc(k),zc(k+1),red(k),green(k),blue(k),
     &        1,1000,iy)
 30   continue
      close(nd)
c
      do 10 k=1,nc-1
         call ngsti(k,nstr)
         dname = fname(1:nglen(fname)) // '.' // nstr(1:nglen(nstr))
         open(nd,file=dname,status='UNKNOWN')
         call ngcf0a(nd,nx,ny,x,y,z,zc(k),zc(k+1),ires,jres)
         close(nd)
 10   continue
c
      return
      end
c
c
c
c
c
c
      subroutine ngcf1s(nd,fname,nx,ny,x,y,z,nc,i1,j1,i2,j2,nmsk,
     &     ires,jres)
c     *****************************************************************
c     *  ngcf1s - Contour Fill No.1, Specified area
c     *   ( zc(n) = zmin+(n-1)*dz, n=1,...,nc, zc(nc)=zmax )
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real          z(nx,ny) : values of grid points
c     *   integer             nc : number of contour levels
c     *   integer    i1,j1,i2,j2 : range of drowing by grid index
c     *   integer    nmsk(nx,ny) : mask array, 0:draw, else:neglect     
c     *   integer      ires,jres : resolutions in x,y-directions    
c     *    
c     *****************************************************************
       dimension x(nx,ny),y(nx,ny),z(nx,ny)
      integer nmsk(nx,ny)
      character*80 fname 
c
      call ngrngs(nx,ny,z,zmin,imin,jmin,zmax,imax,jmax,
     &     i1,j1,i2,j2,nmsk) 
      dz = (zmax-zmin)/float(nc-1)
c
      call ngcf3s(nd,fname,nx,ny,x,y,z,nc,zmin,dz,
     &     i1,j1,i2,j2,nmsk,ires,jres)
c
      return
      end
c
c
c
      subroutine ngcf2s(nd,fname,nx,ny,x,y,z,nc,i1,j1,i2,j2,nmsk,
     &     ires,jres)
c     *****************************************************************
c     *  ngcf2s - Contour Fill No.2, Specified area
c     *   ( zc(n) = zc(n+1)*ratio, zc(nc)=zmax, zc(1)=zmin )  
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real          z(nx,ny) : values of grid points
c     *   integer             nc : number of contour levels
c     *   integer    i1,j1,i2,j2 : range of drowing by grid index
c     *   integer    nmsk(nx,ny) : mask array, 0:draw, else:neglect     
c     *   integer      ires,jres : resolutions in x,y-directions    
c     *    
c     *****************************************************************
      dimension x(nx,ny),y(nx,ny),z(nx,ny)
      integer nmsk(nx,ny)
      character*80 fname 
c
      call ngrngs(nx,ny,z,zmin,imin,jmin,zmax,imax,jmax,
     &     i1,j1,i2,j2,nmsk)
      if (zmin.le.0.0) then
         ratio = 0.1
      else
         a = (log(zmin)-log(zmax))/(float(nc)-1.0)
         ratio = exp(a)
      endif
c
      call ngcf4s(nd,fname,nx,ny,x,y,z,nc,zmax,ratio,i1,j1,i2,j2,nmsk,
     &     ires,jres)
c
      return
      end
c
c
c
      subroutine ngcf3s(nd,fname,nx,ny,x,y,z,nc,zmin,dz,
     &     i1,j1,i2,j2,nmsk,ires,jres)
c     *****************************************************************
c     *  ngcf3s - Contour Fill No.3, Specified area
c     *   ( zc(n) = zmin+(n-1)*dz, n=1,...,nc )
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real          z(nx,ny) : values of grid points
c     *   integer             nc : number of contour levels
c     *   real              zmin : minimum value of contour levels
c     *   real                dz : interval between contour levels 
c     *   integer    i1,i2,j1,j2 : range of drowing by grid index
c     *   integer    nmsk(nx,ny) : mask array, 0:draw, else:neglect     
c     *   integer      ires,jres : resolutions in x,y-directions    
c     *    
c     *****************************************************************
      include 'ng2d_par.f'
      dimension x(nx,ny),y(nx,ny),z(nx,ny),zc(ncmax)
      integer nmsk(nx,ny)
      character*80 fname 
c
      do 10 i=1,nc
         zc(i)=zmin+dz*(i-1)
 10   continue
c
      call ngcf5s(nd,fname,nx,ny,x,y,z,nc,zc,
     &     i1,j1,i2,j2,nmsk,ires,jres)
c
      return
      end
c
c
c
      subroutine ngcf4s(nd,fname,nx,ny,x,y,z,nc,zmax,ratio,
     &     i1,j1,i2,j2,nmsk,ires,jres)
c     *****************************************************************
c     *  ngcf4s - Contour Fill No.4, Specified area
c     *   ( zc(n) = zc(n+1)*ratio, zc(nc)=zmax )
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real          z(nx,ny) : values of grid points
c     *   integer             nc : number of contour levels
c     *   real              zmax : max value of contour levels
c     *   real             ratio : ratio of each contour levels 
c     *   integer    i1,i2,j1,j2 : range of drowing by grid index
c     *   integer    nmsk(nx,ny) : mask array, 0:draw, else:neglect     
c     *   integer      ires,jres : resolutions in x,y-directions    
c     *    
c     *****************************************************************
      include 'ng2d_par.f'
      dimension x(nx,ny),y(nx,ny),z(nx,ny),zc(ncmax)
      integer nmsk(nx,ny)
      character*80 fname 
c
      zc(nc)=zmax
c
      do 10 i=1,nc-1
         zc(nc-i)=zc(nc-i+1)*ratio
 10   continue
c
      call ngcf5s(nd,fname,nx,ny,x,y,z,nc,zc,i1,j1,i2,j2,nmsk,ires,jres)
c
      return
      end
c
c
c
      subroutine ngcf5s(nd,fname,nx,ny,x,y,z,nc,zc,i1,j1,i2,j2,nmsk,
     &     ires,jres)
c     *****************************************************************
c     *  ngcf5s - Contour Fill No.5, Specified area
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real          z(nx,ny) : values of grid points
c     *   integer             nc : number of contour lines
c     *   real             zc(nc) : value of contour lines
c     *   integer    i1,j1,i2,j2 : range of drowing by grid index
c     *   integer    nmsk(nx,ny) : mask array, 0:draw, else:neglect     
c     *   integer      ires,jres : resolutions in x,y-directions    
c     *    
c     *****************************************************************
      include 'ng2d_par.f'
      dimension x(nx,ny),y(nx,ny),z(nx,ny),zc(*)
      integer nmsk(nx,ny)
      character*80 fname,dname,nstr 
      integer red(ncmax),green(ncmax),blue(ncmax)
c
      call ngclm(nc-1,red,green,blue)
c
      dname = fname(1:nglen(fname)) // '.part' 
      open(nd,file=dname,status='UNKNOWN')
      do 30 k=1,nc-1
         call ngsti(k,nstr)
         dname = fname(1:nglen(fname)) // '.' // nstr(1:nglen(nstr))
         iy = 1000+(nc-k)*300
         call ngsrec(nd,dname,zc(k),zc(k+1),red(k),green(k),blue(k),
     &        1,1000,iy)
 30   continue
      close(nd)
c
      do 10 k=1,nc-1
         call ngsti(k,nstr)
         dname = fname(1:nglen(fname)) // '.' // nstr(1:nglen(nstr))
         open(nd,file=dname,status='UNKNOWN')
         call ngcf0s(nd,nx,ny,x,y,z,i1,j1,i2,j2,nmsk,zc(k),zc(k+1),
     &           ires,jres)
         close(nd)
 10   continue
c
      return
      end
c
c
c
      subroutine ngac1a(nd,fname,nx,ny,x,y,u,v,w,fac,ref,id,jd,nc)
c     *****************************************************************
c     *  ngac1a - Arrows with Colors No.1, All area
c     *   ( wc(n) = wmin+(n-1)*dw, n=1,...,nc, wc(nc)=wmax )
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real u(nx,ny),v(nx,ny) : components of vectors
c     *   real          w(nx,ny) : value indicated by colors
c     *   real               fac : factor of arrow length
c     *   real               ref : magnitude of reference vector
c     *   integer          id,jd : skip steps of grids
c     *   integer             nc : number of color levels        
c     *
c     *****************************************************************
      dimension x(nx,ny),y(nx,ny),u(nx,ny),v(nx,ny),w(nx,ny)
      character*80 fname 
c
      call ngrnga(nx,ny,w,wmin,imin,jmin,wmax,imax,jmax) 
      dw = (wmax-wmin)/float(nc-1)
c
      i1=int(float(id)/2)
      j1=int(float(jd)/2)
c
      call ngac3a(nd,fname,nx,ny,x,y,u,v,w,fac,ref,id,jd,nc,wmin,dw)
c
      return
      end
c
c
c
      subroutine ngac2a(nd,fname,nx,ny,x,y,u,v,w,fac,ref,id,jd,nc)
c     *****************************************************************
c     *  ngac2a - Arrows with Colors, No.2, All area
c     *   ( wc(n) = wc(n+1)*ratio, wc(nc)=wmax, wc(1)=zmin )  
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real u(nx,ny),v(nx,ny) : components of vectors
c     *   real          w(nx,ny) : value indicated by colors
c     *   real               fac : factor of arrow length
c     *   real               ref : magnitude of reference vector
c     *   integer          id,jd : skip steps of grids
c     *   integer             nc : number of color levels        
c     *
c     *****************************************************************
      dimension x(nx,ny),y(nx,ny),u(nx,ny),v(nx,ny),w(nx,ny)
      character*80 fname 
c
      call ngrnga(nx,ny,w,wmin,imin,jmin,wmax,imax,jmax) 
      if (wmin.le.0.0) then
         ratio = 0.1
      else
         a = (log(wmin)-log(wmax))/(float(nc)-1.0)
         ratio = exp(a)
      endif
c
      i1=int(float(id)/2)
      j1=int(float(jd)/2)
c
      call ngac4a(nd,fname,nx,ny,x,y,u,v,w,fac,ref,id,jd,nc,wmax,ratio)
c
      return
      end
c
c
c
      subroutine ngac3a(nd,fname,nx,ny,x,y,u,v,w,fac,ref,id,jd,
     &     nc,wmin,dw)
c     *****************************************************************
c     *  ngac3a - Arrows with Colors, No.3, All area
c     *   ( wc(n) = wmin+(n-1)*dz, n=1,...,nc )
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real u(nx,ny),v(nx,ny) : components of vectors
c     *   real          w(nx,ny) : value indicated by colors
c     *   real               fac : factor of arrow length
c     *   real               ref : magnitude of reference vector
c     *   integer          id,jd : skip steps of grids
c     *   integer             nc : number of color levels        
c     *   real              wmin : minimum value of color levels
c     *   real                dw : interval of color levels
c     *
c     *****************************************************************
      include 'ng2d_par.f'
      dimension x(nx,ny),y(nx,ny),u(nx,ny),v(nx,ny),w(nx,ny),wc(ncmax)
      character*80 fname 
c
      do 10 i=1,nc
         wc(i)=wmin+dw*(i-1)
 10   continue
c
      i1=int(float(id)/2)
      j1=int(float(jd)/2)
c
      call ngac5a(nd,fname,nx,ny,x,y,u,v,w,fac,ref,id,jd,nc,wc)
c
      return
      end
c
c
c
      subroutine ngac4a(nd,fname,nx,ny,x,y,u,v,w,fac,ref,id,jd,
     &     nc,wmax,ratio)
c     *****************************************************************
c     *  ngac4a - Arrows with Colors, No.4, All area
c     *   ( wc(n) = wc(n+1)*ratio, wc(nc)=wmax )
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real u(nx,ny),v(nx,ny) : components of vectors
c     *   real          w(nx,ny) : value indicated by colors
c     *   real               fac : factor of arrow length
c     *   real               ref : magnitude of reference vector
c     *   integer          id,jd : skip steps of grids
c     *   integer             nc : number of color levels        
c     *   real              wmax : maximum value of color levels
c     *   real             ratio : ratio between two color levels
c     *
c     *****************************************************************
      include 'ng2d_par.f'
      dimension x(nx,ny),y(nx,ny),u(nx,ny),v(nx,ny),w(nx,ny),wc(ncmax)
      character*80 fname 
c
      wc(nc)=wmax
c
      do 10 i=1,nc-1
         wc(nc-i)=wc(nc-i+1)*ratio
 10   continue
c
      i1=int(float(id)/2)
      j1=int(float(jd)/2)
c
      call ngac5a(nd,fname,nx,ny,x,y,u,v,w,fac,ref,id,jd,nc,wc)
c
      return
      end
c
c
c
      subroutine ngac5a(nd,fname,nx,ny,x,y,u,v,w,fac,ref,id,jd,nc,wc)
c     *****************************************************************
c     *  ngac5a - Arrows with Colors, No.5, All area
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real u(nx,ny),v(nx,ny) : components of vectors
c     *   real          w(nx,ny) : value indicated by colors
c     *   real               fac : factor of arrow length
c     *   real               ref : magnitude of reference vector
c     *   integer          id,jd : skip steps of grids
c     *   real                nc : number of color level
c     *   real            wc(nc) : value of color level
c     *
c     *****************************************************************
      include 'ng2d_par.f'
      dimension x(nx,ny),y(nx,ny),u(nx,ny),v(nx,ny),w(nx,ny),wc(*)
      integer red(ncmax),green(ncmax),blue(ncmax)
      character*80 fname,dname,nstr 
c
      call ngclm(nc-1,red,green,blue)
c
      dname = fname(1:nglen(fname)) // '.part' 
      open(nd,file=dname,status='UNKNOWN')
      do 30 k=1,nc-1
         call ngsti(k,nstr)
         dname = fname(1:nglen(fname)) // '.' // nstr(1:nglen(nstr))
         iy = 23500+(nc-k)*300
         call ngarws(nd,dname,wc(k),wc(k+1),red(k),green(k),blue(k),
     &        1,1000,iy)
 30   continue
      close(nd)
c
      dname = fname(1:nglen(fname)) // '_ref.part'
      open(nd,file=dname,status='UNKNOWN')
      call ngsarw(nd,fac,1000,23000,ref)
      close(nd)
c
      i1=int(float(id)/2)
      j1=int(float(jd)/2)
c
      do 10 k=1,nc-1
         call ngsti(k,nstr)
         dname = fname(1:nglen(fname)) // '.' // nstr(1:nglen(nstr))
         open(nd,file=dname,status='UNKNOWN')
         call ngaca(nd,nx,ny,x,y,u,v,w,id,jd,fac,wc(k),wc(k+1))
         if (k.eq.nc-1) call ngac2(nd,nx,ny,x,y,u,v,w,
     &        id,jd,fac,wc(k+1))
         close(nd)
 10   continue
c
      return
      end
c
c
c
      subroutine ngac1s(nd,fname,nx,ny,x,y,u,v,w,fac,ref,id,jd,nc,
     &     i1,j1,i2,j2,nmsk)
c     *****************************************************************
c     *  ngac1a - Arrows with Colors No.1, Specified area
c     *   ( wc(n) = wmin+(n-1)*dw, n=1,...,nc, wc(nc)=wmax )
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real u(nx,ny),v(nx,ny) : components of vectors
c     *   real          w(nx,ny) : value indicated by colors
c     *   real               fac : factor of arrow length
c     *   real               ref : magnitude of reference vector
c     *   integer          id,jd : skip steps of grids
c     *   integer             nc : number of color levels        
c     *   integer    i1,i2,j1,j2 : range of drowing by grid index
c     *   integer    nmsk(nx,ny) : mask array, 0:draw, else:neglect     
c     *
c     *****************************************************************
      dimension x(nx,ny),y(nx,ny),u(nx,ny),v(nx,ny),w(nx,ny)
      integer nmsk(nx,ny)
      character*80 fname 
c
      call ngrngs(nx,ny,w,wmin,imin,jmin,wmax,imax,jmax,
     &     i1,j1,i2,j2,nmsk) 
      dw = (wmax-wmin)/float(nc-1)
c
      call ngac3s(nd,fname,nx,ny,x,y,u,v,w,fac,ref,id,jd,nc,wmin,dw,
     &     i1,j1,i2,j2,nmsk)
c
      return
      end
c
c
c
      subroutine ngac2s(nd,fname,nx,ny,x,y,u,v,w,fac,ref,id,jd,nc,
     &     i1,j1,i2,j2,nmsk)
c     *****************************************************************
c     *  ngac2a - Arrows with Colors, No.2, Specified area
c     *   ( wc(n) = wc(n+1)*ratio, wc(nc)=wmax, wc(1)=zmin )  
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real u(nx,ny),v(nx,ny) : components of vectors
c     *   real          w(nx,ny) : value indicated by colors
c     *   real               fac : factor of arrow length
c     *   real               ref : magnitude of reference vector
c     *   integer          id,jd : skip steps of grids
c     *   integer             nc : number of color levels        
c     *   integer    i1,j1,i2,j2 : range of drowing by grid index
c     *   integer    nmsk(nx,ny) : mask array, 0:draw, else:neglect     
c     *
c     *****************************************************************
      dimension x(nx,ny),y(nx,ny),u(nx,ny),v(nx,ny),w(nx,ny)
      integer nmsk(nx,ny)
      character*80 fname 
c
      call ngrngs(nx,ny,w,wmin,imin,jmin,wmax,imax,jmax,
     &     i1,j1,i2,j2,nmsk)
      if (wmin.le.0.0) then
         ratio=0.5
      else
         a = (log(wmin)-log(wmax))/(float(nc)-1.0)
         ratio = exp(a)
      endif
c
      call ngac4s(nd,fname,nx,ny,x,y,u,v,w,fac,ref,id,jd,nc,wmax,ratio,
     &     i1,j1,i2,j2,nmsk)
c
      return
      end
c
c
c
      subroutine ngac3s(nd,fname,nx,ny,x,y,u,v,w,fac,ref,id,jd,
     &     nc,wmin,dw,i1,j1,i2,j2,nmsk)
c     *****************************************************************
c     *  ngac3a - Arrows with Colors, No.3, Specified area
c     *   ( wc(n) = wmin+(n-1)*dz, n=1,...,nc )
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real u(nx,ny),v(nx,ny) : components of vectors
c     *   real          w(nx,ny) : value indicated by colors
c     *   real               fac : factor of arrow length
c     *   real               ref : magnitude of reference vector
c     *   integer          id,jd : skip steps of grids
c     *   integer             nc : number of color levels        
c     *   real              wmin : minimum value of color levels
c     *   real                dw : interval of colors levels
c     *   integer    i1,j1,i2,j2 : range of drowing by grid index
c     *   integer    nmsk(nx,ny) : mask array, 0:draw, else:neglect     
c     *
c     *****************************************************************
      include 'ng2d_par.f'
      dimension x(nx,ny),y(nx,ny),u(nx,ny),v(nx,ny),w(nx,ny),wc(ncmax)
      integer nmsk(nx,ny)
      character*80 fname 
c
      do 10 i=1,nc
         wc(i)=wmin+dw*(i-1)
 10   continue
c
      call ngac5s(nd,fname,nx,ny,x,y,u,v,w,fac,ref,id,jd,nc,wc,
     &     i1,j1,i2,j2,nmsk)
c
      return
      end
c
c
c
      subroutine ngac4s(nd,fname,nx,ny,x,y,u,v,w,fac,ref,id,jd,
     &     nc,wmax,ratio,i1,j1,i2,j2,nmsk)
c     *****************************************************************
c     *  ngac4a - Arrows with Colors, No.4, Specified area
c     *   ( wc(n) = wc(n+1)*ratio, wc(nc)=wmax )
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real u(nx,ny),v(nx,ny) : components of vectors
c     *   real          w(nx,ny) : value indicated by colors
c     *   real               fac : factor of arrow length
c     *   real               ref : magnitude of reference vector
c     *   integer          id,jd : skip steps of grids
c     *   integer             nc : number of color levels        
c     *   real              wmax : maximum value of colors levels
c     *   real             ratio : ratio between two color levels
c     *   integer    i1,j1,i2,j2 : range of drowing by grid index
c     *   integer    nmsk(nx,ny) : mask array, 0:draw, else:neglect     
c     *
c     *****************************************************************
      include 'ng2d_par.f'
      dimension x(nx,ny),y(nx,ny),u(nx,ny),v(nx,ny),w(nx,ny),wc(ncmax),
     &     nmsk(nx,ny)
      character*80 fname 
c
      wc(nc)=wmax
c
      do 10 i=1,nc-1
         wc(nc-i)=wc(nc-i+1)*ratio
 10   continue
c
      call ngac5s(nd,fname,nx,ny,x,y,u,v,w,fac,ref,id,jd,nc,wc,
     &     i1,j1,i2,j2,nmsk)
c
      return
      end
c
c
c
      subroutine ngac5s(nd,fname,nx,ny,x,y,u,v,w,fac,ref,id,jd,nc,wc,
     &     i1,j1,i2,j2,nmsk)
c     *****************************************************************
c     *  ngac5a - Arrows with Colors, No.5, Specified area
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real u(nx,ny),v(nx,ny) : components of vectors
c     *   real          w(nx,ny) : value indicated by colors
c     *   real               fac : factor of arrow length
c     *   real               ref : magnitude of reference vector
c     *   integer          id,jd : skip steps of grids
c     *   real                nc : number of color level
c     *   real            wc(nc) : value of color level
c     *   integer    i1,j1,i2,j2 : range of drowing by grid index
c     *   integer    nmsk(nx,ny) : mask array, 0:draw, else:neglect     
c     *
c     *****************************************************************
      include 'ng2d_par.f'
      dimension x(nx,ny),y(nx,ny),u(nx,ny),v(nx,ny),w(nx,ny),wc(*),
     &     nmsk(nx,ny)
      character*80 fname,dname,nstr 
      integer red(ncmax),green(ncmax),blue(ncmax)
c
      call ngclm(nc-1,red,green,blue)
c
      dname = fname(1:nglen(fname)) // '.part' 
      open(nd,file=dname,status='UNKNOWN')
      do 30 k=1,nc-1
         call ngsti(k,nstr)
         dname = fname(1:nglen(fname)) // '.' // nstr(1:nglen(nstr))
         iy = 23500+(nc-k)*300
         call ngarws(nd,dname,wc(k),wc(k+1),red(k),green(k),blue(k),
     &        1,1000,iy)
 30   continue
      close(nd)
c
      dname = fname(1:nglen(fname)) // '_ref.part'
      open(nd,file=dname,status='UNKNOWN')
      call ngsarw(nd,fac,1000,23000,ref)
      close(nd)
c
      do 10 k=1,nc-1
         call ngsti(k,nstr)
         dname = fname(1:nglen(fname)) // '.' // nstr(1:nglen(nstr))
         open(nd,file=dname,status='UNKNOWN')
         call ngacs(nd,nx,ny,x,y,u,v,w,i1,j1,i2,j2,nmsk,id,jd,fac,
     &        wc(k),wc(k+1))
         if (k.eq.nc-1) call ngac1(nd,nx,ny,x,y,u,v,w,i1,j1,i2,j2,nmsk,
     &        id,jd,fac,wc(k+1))
         close(nd)
 10   continue
c
      return
      end
c
c
c
      subroutine ngama(nd,fname,nx,ny,x,y,u,v,fac,ref,id,jd)
c     *****************************************************************
c     *  ngama - Arrows without Colors, All area  
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real u(nx,ny),v(nx,ny) : components of vectors
c     *   real               fac : factor of arrow length
c     *   real               ref : magnitude of reference vector
c     *   integer          id,jd : skip steps of grids
c     *
c     *****************************************************************
      dimension x(nx,ny),y(nx,ny),u(nx,ny),v(nx,ny)
      character*80 fname,dstr 
c
      dstr = fname(1:nglen(fname)) // '.part'
      open(nd,file=dstr,status='UNKNOWN')
      dstr='new file'
      write(nd,'(A8)') dstr
      dstr= 'file::file=''' // fname(1:nglen(fname)) // '.dat'''
      write(nd,*) dstr(1:nglen(dstr)) 
      dstr= 'file::x=1'
      write(nd,*) dstr(1:nglen(dstr))
      dstr= 'file::y=5'
      write(nd,*) dstr(1:nglen(dstr))
      dstr= 'file::type=arrow'
      write(nd,*) dstr(1:nglen(dstr))
      close(nd)
c
      dstr = fname(1:nglen(fname)) // '_ref.part'
      open(nd,file=dstr,status='UNKNOWN')
      call ngsarw(nd,fac,1000,23000,ref)
      close(nd)
c
      i1 = id/2
      j1 = jd/2
c
      dstr = fname(1:nglen(fname)) // '.dat'
      open(nd,file=dstr,status='UNKNOWN')
      write(nd,'(A45)') '#     x            y            u           v'
      do 10 j=j1,ny,jd
         do 10 i=i1,nx,id
            write(nd,100) x(i,j),y(i,j),u(i,j),v(i,j),
     &           x(i,j)+fac*u(i,j),y(i,j)+fac*v(i,j)
 10   continue
      close(nd)
c
 100  format(6(1pe13.5))
c
      return
      end
c
c
c
      subroutine ngams(nd,fname,nx,ny,x,y,u,v,fac,ref,id,jd,
     &     i1,j1,i2,j2,nmsk)
c     *****************************************************************
c     *  ngams - Arrows without Colors, Specified area
c     *
c     *   integer             nd : I/O unit
c     *   character*80     fname : prefix of filename
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real u(nx,ny),v(nx,ny) : components of vectors
c     *   real               fac : factor of arrow length
c     *   real               ref : magnitude of reference vector
c     *   integer          id,jd : skip steps of grids
c     *   integer    i1,j1,i2,j2 : range of drowing by grid index
c     *   integer    nmsk(nx,ny) : mask array, 0:draw, else:neglect     
c     *
c     *****************************************************************
      dimension x(nx,ny),y(nx,ny),u(nx,ny),v(nx,ny),nmsk(nx,ny)
      character*80 fname,dstr
c
      dstr = fname(1:nglen(fname)) // '.part'
      open(nd,file=dstr,status='UNKNOWN')
      dstr='new file'
      write(nd,'(A8)') dstr
      dstr= 'file::file=''' // fname(1:nglen(fname)) // '.dat'''
      write(nd,*) dstr(1:nglen(dstr)) 
      dstr= 'file::x=1'
      write(nd,*) dstr(1:nglen(dstr))
      dstr= 'file::y=5'
      write(nd,*) dstr(1:nglen(dstr))
      dstr= 'file::type=arrow'
      write(nd,*) dstr(1:nglen(dstr))
      close(nd)
c
      dstr = fname(1:nglen(fname)) // '_ref.part'
      open(nd,file=dstr,status='UNKNOWN')
      call ngsarw(nd,fac,1000,23000,ref)
      close(nd)
c
      dstr = fname(1:nglen(fname)) // '.dat'
      open(nd,file=dstr,status='UNKNOWN')
      write(nd,'(A45)') '#     x            y            u           v'
      do 10 j=j1,j2,jd
         do 10 i=i1,i2,id
            if (nmsk(i,j).ne.0) goto 10
            write(nd,100) x(i,j),y(i,j),u(i,j),v(i,j),
     &           x(i,j)+fac*u(i,j),y(i,j)+fac*v(i,j)
 10   continue
      close(nd)
c
 100  format(6(1pe13.5))
      return
      end
c
c
c
      subroutine ngslin(nd,fname,zval,ir,ig,ib,ileg,ix,iy)
c     *****************************************************************
c     * ngslin  -- output Ngraph script for a line
c     *
c     *   integer         nd : I/O unit
c     *   character*80 fname : name of data file
c     *   real          zval : value indicated as legend
c     *   integer   ir,ig,ib : color (red,green,blue 0:255)
c     *   integer       ileg : 1: with legend, 0: without legend
c     *   integer      ix,iy : position of legend
c     *
c     *****************************************************************
      character*80 dstr,nstr,fname
c     --- file information
      dstr='new file'
      write(nd,'(A8)') dstr
      dstr= 'file::file=''' // fname(1:nglen(fname)) // ''''
      write(nd,*) dstr(1:nglen(dstr)) 
      dstr= 'file::x=1'
      write(nd,*) dstr(1:nglen(dstr))
      dstr= 'file::y=2'
      write(nd,*) dstr(1:nglen(dstr))
      dstr= 'file::type=line'
      write(nd,*) dstr(1:nglen(dstr))
      call ngsti(ir,dstr)
      dstr = 'file::R=' // dstr
      write(nd,*) dstr(1:nglen(dstr))
      call ngsti(ig,dstr)
      dstr = 'file::G=' // dstr
      write(nd,*) dstr(1:nglen(dstr))
      call ngsti(ib,dstr)
      dstr = 'file::B=' // dstr
      write(nd,*) dstr(1:nglen(dstr))
c     --- legend information
      if (ileg.eq.1) then
c        --- text
         dstr='new text'
         write(nd,'(A8)') dstr
         call ngsti(ix+1000,dstr)
         dstr='text::x=' // dstr
         write(nd,*) dstr(1:nglen(dstr))
         call ngsti(iy,dstr)
         dstr = 'text::y=' // dstr
         write(nd,*) dstr(1:nglen(dstr))
         dstr = 'text::pt=800'
         write(nd,*) dstr(1:nglen(dstr))
         nstr = ' text::text='''
         dstr = ''''
         write(nd,'(A13,1pe13.5,A1)') nstr,zval,dstr
         call ngsti(ir,dstr)
         dstr = 'text::R=' // dstr
         write(nd,*) dstr(1:nglen(dstr))
         call ngsti(ig,dstr)
         dstr = 'text::G=' // dstr
         write(nd,*) dstr(1:nglen(dstr))
         call ngsti(ib,dstr)
         dstr = 'text::B=' // dstr
         write(nd,*) dstr(1:nglen(dstr))
c        --- line
         dstr='new line'
         write(nd,'(A8)') dstr
         call ngsti(ir,dstr)
         dstr = 'line::R=' // dstr
         write(nd,*) dstr(1:nglen(dstr))
         call ngsti(ig,dstr)
         dstr = 'line::G=' // dstr
         write(nd,*) dstr(1:nglen(dstr))
         call ngsti(ib,dstr)
         dstr = 'line::B=' // dstr
         write(nd,*) dstr(1:nglen(dstr))
         dstr = 'line::arrow=none'
         write(nd,*) dstr(1:nglen(dstr))
         dstr = ' line::points='''
         ix1=ix
         iy1=iy-80
         ix2=ix+800
         iy2=iy-80
         write(nd,'(A15,I6,I6,I6,I6,A1)') dstr,ix1,iy1,ix2,iy2,''''
      endif
      return
      end
c
c
c
      subroutine ngclm(nc,red,green,blue)
c     *****************************************************************
c     * ngclm  -- generate color map
c     *
c     *   integer         nc : depth of colors
c     *   integer red(nc),green(nc),brue(nc)
c     *                      : color table
c     *
c     *****************************************************************
      integer red(*),green(*),blue(*),nc
c
      if (nc.eq.1) then
         red(1)=0
         green(1)=255
         blue(1)=0
      elseif (nc.eq.2) then
         red(1)=0
         green(1)=0
         blue(1)=255
         red(2)=255
         green(2)=0
         blue(2)=0
      elseif (nc.eq.3) then
         red(1)=0
         green(1)=0
         blue(1)=255
         red(2)=0
         green(2)=255
         blue(2)=0
         red(3)=255
         green(3)=0
         blue(3)=0
      elseif (nc.eq.4) then
         red(1)=0
         green(1)=0
         blue(1)=255
         red(2)=0
         green(2)=255
         blue(2)=255
         red(3)=255
         green(3)=255
         blue(3)=0
         red(4)=255
         green(4)=0
         blue(4)=0
      else
         c = 1020.0/float(nc-1)
         cnc1 = float(nc-1)*0.25
         cnc2 = float(nc-1)*0.5
         cnc3 = float(nc-1)*0.75
         n1=int(cnc1)+1
         n2=int(cnc2)+1
         n3=int(cnc3)+1
         do 10 i=1,n1
            red(i)=0
            green(i)=c*(i-1)
            blue(i)=255
 10      continue
         do 20 i=n1+1,n2
            red(i)=0
            green(i)=255
            blue(i)=510-c*(i-1)
 20      continue
         do 30 i=n2+1,n3
            red(i)=c*(i-1)-510
            green(i)=255
            blue(i)=0
 30      continue
         do 40 i=n3+1,nc
            red(i)=255
            green(i)=1020-c*(i-1)
            blue(i)=0
 40      continue
      endif
c     
      return
      end
c
c
c
      subroutine ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,tri,zval,
     &     ins,is,js,eps)
c     *****************************************************************
c     * ngtcl  -- trace and output cordinate of points on the line
c     *
c     *   integer              nd : I/O unit
c     *   integer           nx,ny : size of array
c     *   real  x(nx,ny),y(nx,ny) : positions of grid points
c     *   real           z(nx,ny) : values of grid points
c     *   integer     i1,j1,i2,j2 : range of drowing by grid index
c     *   integer           mx,my : size of flag array
c     *   logical*1 tri(0,mx,0:my): flag of triangle domain
c     *   real               zval : contour level
c     *   integer       ins,is,js : index of start segment    
c     *   real                eps : lower limit of denominator in
c     *                             calculations of position
c     *
c     *****************************************************************
      dimension x(nx,ny),y(nx,ny),z(nx,ny)
      logical*1 tri(4,0:mx,0:my),tria,trib
c
c     --- if zz in [z1,z2], f(zz,z1,z2)<0
      f(zz,z1,z2)=(z1-zz)*(z2-zz)
c
c     *** check start position (ins,is,js)  
c
      in = ins
      i = is
      j = js
      ia = 0
c
      xc = 0.25*(x(i,j)+x(i+1,j)+x(i+1,j+1)+x(i,j+1))
      yc = 0.25*(y(i,j)+y(i+1,j)+y(i+1,j+1)+y(i,j+1))
      zc = 0.25*(z(i,j)+z(i+1,j)+z(i+1,j+1)+z(i,j+1))
c
      goto (10,20,30,40,50,60,70,80),in
      goto 1000
c
 10   tria = tri(3,i,j-1)
      trib = tri(1,i,j)
      if (tria.and.trib) goto 1000
      x1 = x(i+1,j)
      y1 = y(i+1,j)
      z1 = z(i+1,j)
      x2 = x(i,j)
      y2 = y(i,j)
      z2 = z(i,j)
c      if ((i.eq.41).and.(j.eq.12)) write(*,*) x1,y1,z1,x2,y2,z2
      if (f(zval,z1,z2).ge.0.0) goto 1000
      goto 310
c
 20   tria = tri(4,i,j)
      trib = tri(2,i-1,j)
      if (tria.and.trib) goto 1000
      x1 = x(i,j+1)
      y1 = y(i,j+1)
      z1 = z(i,j+1)
      x2 = x(i,j)
      y2 = y(i,j)
      z2 = z(i,j)
      if (f(zval,z1,z2).ge.0.0) goto 1000
      goto 310
c
 30   tria = tri(4,i,j)
      trib = tri(1,i,j)
      if (tria.and.trib) goto 1000
      x1 = x(i,j)
      y1 = y(i,j)
      z1 = z(i,j)
      x2 = 0.25*(x(i,j)+x(i+1,j)+x(i+1,j+1)+x(i,j+1))
      y2 = 0.25*(y(i,j)+y(i+1,j)+y(i+1,j+1)+y(i,j+1))
      z2 = 0.25*(z(i,j)+z(i+1,j)+z(i+1,j+1)+z(i,j+1))
      if (f(zval,z1,z2).ge.0.0) goto 1000
      goto 310
c
 40   tria = tri(1,i,j)
      trib = tri(2,i,j)
      if (tria.and.trib) goto 1000
      x1 = x(i+1,j)
      y1 = y(i+1,j)
      z1 = z(i+1,j)
      x2 = 0.25*(x(i,j)+x(i+1,j)+x(i+1,j+1)+x(i,j+1))
      y2 = 0.25*(y(i,j)+y(i+1,j)+y(i+1,j+1)+y(i,j+1))
      z2 = 0.25*(z(i,j)+z(i+1,j)+z(i+1,j+1)+z(i,j+1))
      if (f(zval,z1,z2).ge.0.0) goto 1000
      goto 310
c
 50   tria = tri(3,i,j)
      trib = tri(4,i,j)
      if (tria.and.trib) goto 1000
      x1 = x(i,j+1)
      y1 = y(i,j+1)
      z1 = z(i,j+1)
      x2 = 0.25*(x(i,j)+x(i+1,j)+x(i+1,j+1)+x(i,j+1))
      y2 = 0.25*(y(i,j)+y(i+1,j)+y(i+1,j+1)+y(i,j+1))
      z2 = 0.25*(z(i,j)+z(i+1,j)+z(i+1,j+1)+z(i,j+1))
      if (f(zval,z1,z2).ge.0.0) goto 1000
      goto 310
c
 60   tria = tri(2,i,j)
      trib = tri(3,i,j)
      if (tria.and.trib) goto 1000
      x1 = x(i+1,j+1)
      y1 = y(i+1,j+1)
      z1 = z(i+1,j+1)
      x2 = 0.25*(x(i,j)+x(i+1,j)+x(i+1,j+1)+x(i,j+1))
      y2 = 0.25*(y(i,j)+y(i+1,j)+y(i+1,j+1)+y(i,j+1))
      z2 = 0.25*(z(i,j)+z(i+1,j)+z(i+1,j+1)+z(i,j+1))
      if (f(zval,z1,z2).ge.0.0) goto 1000
      goto 310
c
 70   ia = 0
      xx = x(i,j)
      yy = y(i,j)
      if (zval.ne.z(i,j)) goto 1000
c
      if (tri(1,i,j)) goto 71
      z1 = z(i+1,j)
      z2 = 0.25*(z(i,j)+z(i+1,j)+z(i+1,j+1)+z(i,j+1))
      if (zval.eq.z1) then
         if (tri(3,i,j-1)) goto 71
         z0 = 0.25*(z(i,j-1)+z(i+1,j-1)+z(i+1,j)+z(i,j))
         ib = 1
         if (zval.gt.z0) goto 330
      endif
      if (z1.ge.z2) goto 71
      ib = 2
      if (f(zval,z1,z2).lt.0.0) goto 330
      ib = 3
      if (zval.eq.z2) goto 330         
c
 71   if (tri(4,i,j)) goto 72
      z1 = 0.25*(z(i,j)+z(i+1,j)+z(i+1,j+1)+z(i,j+1))
      z2 = z(i,j+1)
      if (z1.ge.z2) goto 72
      ib = 4
      if (f(zval,z1,z2).lt.0.0) goto 330
      ib = 5
      if (zval.eq.z2) goto 330
c
 72   if (tri(2,i-1,j)) goto 73
      z1 = z(i,j+1)
      z2 = 0.25*(z(i-1,j)+z(i,j)+z(i,j+1)+z(i-1,j+1))
      if (z1.ge.z2) goto 73
      ib = 6
      if (f(zval,z1,z2).lt.0.0) goto 330
      ib = 7
      if (zval.eq.z2) goto 330
c
 73   if (tri(1,i-1,j)) goto 74
      z1 = 0.25*(z(i-1,j)+z(i,j)+z(i,j+1)+z(i-1,j+1))
      z2 = z(i-1,j)
      if (z1.ge.z2) goto 74
      ib = 8
      if (f(zval,z1,z2).lt.0.0) goto 330
      ib = 9
      if (zval.eq.z2) goto 330
c
 74   if (tri(3,i-1,j-1)) goto 75
      z1 = z(i-1,j)
      z2 = 0.25*(z(i-1,j-1)+z(i,j-1)+z(i,j)+z(i-1,j))
      if (z1.ge.z2) goto 75
      ib = 10
      if (f(zval,z1,z2).lt.0.0) goto 330
      ib = 11
      if (zval.eq.z2) goto 330
c
 75   if (tri(2,i-1,j-1)) goto 76
      z1 = 0.25*(z(i-1,j-1)+z(i,j-1)+z(i,j)+z(i-1,j))
      z2 = z(i,j-1)
      if (z1.ge.z2) goto 76
      ib = 12
      if (f(zval,z1,z2).lt.0.0) goto 330
      ib = 13
      if (zval.eq.z2) goto 330
c
 76   if (tri(4,i,j-1)) goto 77
      z1 = z(i,j-1)
      z2 = 0.25*(z(i,j-1)+z(i+1,j-1)+z(i+1,j)+z(i,j))
      if (z1.ge.z2) goto 77
      ib = 14
      if (f(zval,z1,z2).lt.0.0) goto 330
      ib = 15
      if (zval.eq.z2) goto 330
c
 77   if (tri(3,i,j-1)) goto 1000
      z1 = 0.25*(z(i,j-1)+z(i+1,j-1)+z(i+1,j)+z(i,j))
      z2 = z(i+1,j)
      if (z1.ge.z2) goto 1000
      ib = 16
      if (f(zval,z1,z2).lt.0.0) goto 330
      ib = 1
      if (zval.eq.z2) goto 330
      goto 1000 
c
 80   ia = 0
      xx = 0.25*(x(i,j)+x(i+1,j)+x(i+1,j+1)+x(i,j+1))
      yy = 0.25*(y(i,j)+y(i+1,j)+y(i+1,j+1)+y(i,j+1))
      zz = 0.25*(z(i,j)+z(i+1,j)+z(i+1,j+1)+z(i,j+1))
      if (zval.ne.zz) goto 1000
c
      if (tri(1,i,j)) goto 81
      z1=z(i,j)
      z2=z(i+1,j) 
      if (z1.ge.z2) goto 81
      if (zval.eq.z1) then
         if (tri(4,i,j)) goto 81
         ib = 1
         if (zval.gt.z(i,j+1)) goto 330
      endif
      ib = 2
      if (f(zval,z1,z2).lt.0.0) goto 330
      ib = 3
      if (zval.eq.z2) goto 330
c
 81   if (tri(2,i,j)) goto 82
      z1=z(i+1,j)
      z2=z(i+1,j+1) 
      if (z1.ge.z2) goto 82
      ib = 4
      if (f(zval,z1,z2).lt.0.0) goto 330
      ib = 5
      if (zval.eq.z2) goto 330
c
 82   if (tri(3,i,j)) goto 83
      z1=z(i+1,j+1)
      z2=z(i,j+1)
      if (z1.ge.z2) goto 83
      ib = 6
      if (f(zval,z1,z2).lt.0.0) goto 330
      ib = 7
      if (zval.eq.z2) goto 330
c
 83   if (tri(4,i,j)) goto 1000
      z1=z(i,j+1)
      z2=z(i,j)
      if (z1.ge.z2) goto 1000
      ib = 8
      if (f(zval,z1,z2).lt.0.0) goto 330
      ib = 1
      if (zval.eq.z2) goto 330
c
      goto 1000
c
 310  if (trib) goto 315
      if (zval.gt.z1) goto 320
 315  if (tria) goto 1000
      if (zval.gt.z2) goto 320
      goto 1000
c
 320  dz = z2-z1
      if (abs(dz).lt.eps) then
         xx = 0.5*(x1+x2)
         yy = 0.5*(y1+y2)
      else
         dz1 = (zval-z1)/dz
         dz2 = (zval-z2)/dz  
         xx = dz1*x2-dz2*x1
         yy = dz1*y2-dz2*y1
      endif
c
 330  write(nd,'(2E13.5)') xx,yy
c
c     --- find next segment
c
 100  xc = 0.25*(x(i,j)+x(i+1,j)+x(i+1,j+1)+x(i,j+1))
      yc = 0.25*(y(i,j)+y(i+1,j)+y(i+1,j+1)+y(i,j+1))
      zc = 0.25*(z(i,j)+z(i+1,j)+z(i+1,j+1)+z(i,j+1))
c
      goto (110,120,130,140,150,160,170,180),in
c
 110  if (tri(1,i,j)) goto 115
      if (z(i,j).le.z(i+1,j)) goto 115
      tri(1,i,j)=.true. 
      x1 = x(i+1,j)
      y1 = y(i+1,j)
      z1 = z(i+1,j)
      x2 = 0.25*(x(i,j)+x(i+1,j)+x(i+1,j+1)+x(i,j+1))
      y2 = 0.25*(y(i,j)+y(i+1,j)+y(i+1,j+1)+y(i,j+1))
      z2 = 0.25*(z(i,j)+z(i+1,j)+z(i+1,j+1)+z(i,j+1))
      x3 = x(i,j)
      y3 = y(i,j)
      z3 = z(i,j)
      if (f(zval,z1,z2).lt.0.0) goto 111
      if (zval.eq.z2) goto 112
      if (f(zval,z2,z3).lt.0.0) goto 113
      goto 300
 111  in = 4
      goto 400
 112  in = 8
      ia = 2
      xx = x2
      yy = y2
      goto 420
 113  in = 3
      x1 = x3
      y1 = y3
      z1 = z3
      goto 400
c
 115  if (tri(3,i,j-1)) goto 300
      if (z(i+1,j).le.z(i,j)) goto 300
      tri(3,i,j-1)=.true.
      x1 = x(i,j)
      y1 = y(i,j)
      z1 = z(i,j)
      x2 = 0.25*(x(i,j-1)+x(i+1,j-1)+x(i+1,j)+x(i,j))
      y2 = 0.25*(y(i,j-1)+y(i+1,j-1)+y(i+1,j)+y(i,j))
      z2 = 0.25*(z(i,j-1)+z(i+1,j-1)+z(i+1,j)+z(i,j))
      x3 = x(i+1,j)
      y3 = y(i+1,j)
      z3 = z(i+1,j)
      if (f(zval,z1,z2).lt.0.0) goto 116
      if (zval.eq.z2) goto 117
      if (f(zval,z2,z3).lt.0.0) goto 118
      goto 300
 116  in = 5
      j = j-1
      goto 400
 117  in = 8
      ia = 6
      j = j-1
      xx = x2
      yy = y2
      goto 420
 118  in = 6
      j = j-1
      x1 = x3
      y1 = y3
      z1 = z3
      goto 400
c
 120  if (tri(2,i-1,j)) goto 125
      if (z(i,j).le.z(i,j+1)) goto 125
      tri(2,i-1,j)=.true.
      x1 = x(i,j+1)
      y1 = y(i,j+1)
      z1 = z(i,j+1)
      x2 = 0.25*(x(i-1,j)+x(i,j)+x(i,j+1)+x(i-1,j+1))
      y2 = 0.25*(y(i-1,j)+y(i,j)+y(i,j+1)+y(i-1,j+1))
      z2 = 0.25*(z(i-1,j)+z(i,j)+z(i,j+1)+z(i-1,j+1))
      x3 = x(i,j)
      y3 = y(i,j)
      z3 = z(i,j)
      if (f(zval,z1,z2).lt.0.0) goto 121
      if (zval.eq.z2) goto 122
      if (f(zval,z2,z3).lt.0.0) goto 123
      goto 300
 121  in = 6
      i = i-1
      goto 400
 122  in = 8
      ia = 4
      i = i-1
      xx = x2
      yy = y2
      goto 420
 123  in = 4
      i = i-1
      x1 = x3
      y1 = y3
      z1 = z3
      goto 400
c
 125  if (tri(4,i,j)) goto 300
      if (z(i,j+1).le.z(i,j)) goto 300
      tri(4,i,j)=.true.
      x1 = x(i,j)
      y1 = y(i,j)
      z1 = z(i,j)
      x2 = 0.25*(x(i,j)+x(i+1,j)+x(i+1,j+1)+x(i,j+1))
      y2 = 0.25*(y(i,j)+y(i+1,j)+y(i+1,j+1)+y(i,j+1))
      z2 = 0.25*(z(i,j)+z(i+1,j)+z(i+1,j+1)+z(i,j+1))
      x3 = x(i,j+1)
      y3 = y(i,j+1)
      z3 = z(i,j+1)
      if (f(zval,z1,z2).lt.0.0) goto 126
      if (zval.eq.z2) goto 127
      if (f(zval,z2,z3).lt.0.0) goto 128
      goto 300
 126  in = 3
      goto 400
 127  in = 8
      ia = 8
      xx = x2
      yy = y2
      goto 420
 128  in = 5
      x1 = x3
      y1 = y3
      z1 = z3
      goto 400
c
 130  if (tri(1,i,j)) goto 135
      if (zc.le.z(i,j)) goto 135
      tri(1,i,j)=.true.
      x1 = x(i,j)
      y1 = y(i,j)
      z1 = z(i,j)
      x2 = x(i+1,j)
      y2 = y(i+1,j)
      z2 = z(i+1,j)
      x3 = xc
      y3 = yc
      z3 = zc
      if (f(zval,z1,z2).lt.0.0) goto 131
      if (zval.eq.z2) goto 132
      if (f(zval,z2,z3).lt.0.0) goto 133
      goto 300
 131  in = 1
      goto 400
 132  in = 7
      ia = 8
      i = i+1
      xx = x2
      yy = y2
      goto 420
 133  in = 4
      x1 = x3
      y1 = y3
      z1 = z3
      goto 400
c
 135  if (tri(4,i,j)) goto 300
      if (z(i,j).le.zc) goto 300
      tri(4,i,j)=.true.
      x1 = xc
      y1 = yc
      z1 = zc
      x2 = x(i,j+1)
      y2 = y(i,j+1)
      z2 = z(i,j+1)
      x3 = x(i,j)
      y3 = y(i,j)
      z3 = z(i,j)
      if (f(zval,z1,z2).lt.0.0) goto 136
      if (zval.eq.z2) goto 137
      if (f(zval,z2,z3).lt.0.0) goto 138
      goto 300
 136  in = 5
      goto 400
 137  in = 7
      ia = 14
      j = j+1
      xx = x2
      yy = y2
      goto 420
 138  in = 2
      x1 = x3
      y1 = y3
      z1 = z3
      goto 400
c
 140  if (tri(2,i,j)) goto 145
      if (zc.le.z(i+1,j)) goto 145
      tri(2,i,j)=.true.
      x1 = x(i+1,j)
      y1 = y(i+1,j)
      z1 = z(i+1,j)
      x2 = x(i+1,j+1)
      y2 = y(i+1,j+1)
      z2 = z(i+1,j+1)
      x3 = xc
      y3 = yc
      z3 = zc
      if (f(zval,z1,z2).lt.0.0) goto 141
      if (zval.eq.z2) goto 142
      if (f(zval,z2,z3).lt.0.0) goto 143
      goto 300
 141  in = 2
      i = i+1
      goto 400
 142  in = 7
      ia = 12
      i = i+1
      j = j+1
      xx = x2
      yy = y2
      goto 420
 143  in = 6
      x1 = x3
      y1 = y3
      z1 = z3
      goto 400
c
 145  if (tri(1,i,j)) goto 300
      if (z(i+1,j).le.zc) goto 300
      tri(1,i,j)=.true.
      x1 = xc
      y1 = yc
      z1 = zc
      x2 = x(i,j)
      y2 = y(i,j)
      z2 = z(i,j)
      x3 = x(i+1,j)
      y3 = y(i+1,j)
      z3 = z(i+1,j)
      if (f(zval,z1,z2).lt.0.0) goto 146
      if (zval.eq.z2) goto 147
      if (f(zval,z2,z3).lt.0.0) goto 148
      goto 300
 146  in = 3
      goto 400
 147  in = 7
      ia = 2
      xx = x2
      yy = y2
      goto 420
 148  in = 1
      x1 = x3
      y1 = y3
      z1 = z3
      goto 400
c
 150  if (tri(4,i,j)) goto 155
      if (zc.le.z(i,j+1)) goto 155
      tri(4,i,j)=.true.
      x1 = x(i,j+1)
      y1 = y(i,j+1)
      z1 = z(i,j+1)
      x2 = x(i,j)
      y2 = y(i,j)
      z2 = z(i,j)
      x3 = xc
      y3 = yc
      z3 = zc
      if (f(zval,z1,z2).lt.0.0) goto 151
      if (zval.eq.z2) goto 152
      if (f(zval,z2,z3).lt.0.0) goto 153
      goto 300
 151  in = 2
      goto 400
 152  in = 7
      ia = 4
      xx = x2
      yy = y2
      goto 420
 153  in = 3
      x1 = x3
      y1 = y3
      z1 = z3
      goto 400
c
 155  if (tri(3,i,j)) goto 300
      if (z(i,j+1).le.zc) goto 300
      tri(3,i,j)=.true.
      x1 = xc
      y1 = yc
      z1 = zc
      x2 = x(i+1,j+1)
      y2 = y(i+1,j+1)
      z2 = z(i+1,j+1)
      x3 = x(i,j+1)
      y3 = y(i,j+1)
      z3 = z(i,j+1)
      if (f(zval,z1,z2).lt.0.0) goto 156
      if (zval.eq.z2) goto 157
      if (f(zval,z2,z3).lt.0.0) goto 158
      goto 300
 156  in =6
      goto 400
 157  in = 7
      ia = 10
      i = i+1
      j = j+1
      xx = x2
      yy = y2
      goto 420
 158  in = 1
      j = j+1
      x1 = x3
      y1 = y3
      z1 = z3
      goto 400
c
 160  if (tri(3,i,j)) goto 165
      if (zc.le.z(i+1,j+1)) goto 165
      tri(3,i,j)=.true.
      x1 = x(i+1,j+1)
      y1 = y(i+1,j+1)
      z1 = z(i+1,j+1)
      x2 = x(i,j+1)
      y2 = y(i,j+1)
      z2 = z(i,j+1)
      x3 = xc
      y3 = yc
      z3 = zc
      if (f(zval,z1,z2).lt.0.0) goto 161
      if (zval.eq.z2) goto 162
      if (f(zval,z2,z3).lt.0.0) goto 163
      goto 300
 161  in = 1
      j = j+1
      goto 400
 162  in = 7
      ia = 16
      j = j+1
      xx = x2
      yy = y2
      goto 420
 163  in = 5
      x1 = x3
      y1 = y3
      z1 = z3
      goto 400
c
 165  if (tri(2,i,j)) goto 300
      if (z(i+1,j+1).le.zc) goto 300
      tri(2,i,j)=.true.
      x1 = xc
      y1 = yc
      z1 = zc
      x2 = x(i+1,j)
      y2 = y(i+1,j)
      z2 = z(i+1,j)
      x3 = x(i+1,j+1)
      y3 = y(i+1,j+1)
      z3 = z(i+1,j+1)
      if (f(zval,z1,z2).lt.0.0) goto 166
      if (zval.eq.z2) goto 167
      if (f(zval,z2,z3).lt.0.0) goto 168
      goto 300
 166  in = 4
      goto 400
 167  in = 7
      ia = 6
      i = i+1
      xx = x2
      yy = y2
      goto 420
 168  in = 2
      i = i+1
      x1 = x3
      y1 = y3
      z1 = z3
      goto 400
c
 170  if (ia.eq.0) then
         goto (751,752,753,754,755,756,757,758,759,760,
     &        761,762,763,764,765,766),ib
      endif
c
      ic = 0
      if ((ia.eq.1).or.(ia.eq.2)) goto 172
      if ((ia.eq.3).or.(ia.eq.4)) goto 173
      if ((ia.eq.5).or.(ia.eq.6)) goto 174
      if ((ia.eq.7).or.(ia.eq.8)) goto 175
      if ((ia.eq.9).or.(ia.eq.10)) goto 176
      if ((ia.eq.11).or.(ia.eq.12)) goto 177
      if ((ia.eq.13).or.(ia.eq.14)) goto 178
      if ((ia.eq.15).or.(ia.eq.16)) goto 171
c
c     --- 2 ---
 171  if (ic.ge.7) goto 300
      ic = ic + 1
      if (tri(1,i,j)) goto 172
      z1 = z(i+1,j)
      z2 = 0.25*(z(i,j)+z(i+1,j)+z(i+1,j+1)+z(i,j+1))
      if (z1.ge.zval) goto 300
      if (z2.eq.zval) goto 753
      if (z2.ge.zval) goto 752
c
c     --- 4 ---
 172  if (ic.ge.7) goto 300
      ic = ic + 1
      if (tri(4,i,j)) goto 173
      z1 = 0.25*(z(i,j)+z(i+1,j)+z(i+1,j+1)+z(i,j+1))
      z2 = z(i,j+1)
      if (z1.ge.zval) goto 300
      if (z2.eq.zval) goto 755
      if (z2.ge.zval) goto 754
c
c     --- 6 ---
 173  if (ic.ge.7) goto 300
      ic = ic + 1
      if (tri(2,i-1,j)) goto 174
      z1 = z(i,j+1)
      z2 = 0.25*(z(i-1,j)+z(i,j)+z(i,j+1)+z(i-1,j+1))
      if (z1.ge.zval) goto 300
      if (z2.eq.zval) goto 757
      if (z2.ge.zval) goto 756
c
c     --- 8 ---
 174  if (ic.ge.7) goto 300
      ic = ic + 1
      if (tri(1,i-1,j)) goto 175
      z1 = 0.25*(z(i-1,j)+z(i,j)+z(i,j+1)+z(i-1,j+1))
      z2 = z(i-1,j)
      if (z1.ge.zval) goto 300
      if (z2.eq.zval) goto 759
      if (z2.ge.zval) goto 758
c
c     --- 10 ---
 175  if (ic.ge.7) goto 300
      ic = ic + 1
      if (tri(3,i-1,j-1)) goto 176
      z1 = z(i-1,j)
      z2 = 0.25*(z(i-1,j-1)+z(i,j-1)+z(i,j)+z(i-1,j))
      if (z1.ge.zval) goto 300
      if (z2.eq.zval) goto 761
      if (z2.ge.zval) goto 760
c
c     --- 12 ---
 176  if (ic.ge.7) goto 300
      ic = ic + 1
      if (tri(2,i-1,j-1)) goto 177 
      z1 = 0.25*(z(i-1,j-1)+z(i,j-1)+z(i,j)+z(i-1,j))
      z2 = z(i,j-1)
      if (z1.ge.zval) goto 300
      if (z2.eq.zval) goto 763
      if (z2.ge.zval) goto 762
c
c     --- 14 ---
 177  if (ic.ge.7) goto 300
      ic = ic + 1
      if (tri(4,i,j-1)) goto 178
      z1 = z(i,j-1)
      z2 = 0.25*(z(i,j-1)+z(i+1,j-1)+z(i+1,j)+z(i,j))
      if (z1.ge.zval) goto 300
      if (z2.eq.zval) goto 765
      if (z2.ge.zval) goto 764
c
c     --- 16 ---
 178  if (ic.ge.7) goto 300
      ic = ic + 1
      if (tri(3,i,j-1)) goto 171
      z1 = 0.25*(z(i,j-1)+z(i+1,j-1)+z(i+1,j)+z(i,j))
      z2 = z(i+1,j)
      if (z1.ge.zval) goto 300
      if (z2.eq.zval) goto 751
      if (z2.ge.zval) goto 766
      goto 171
c
c
 180  if (ia.eq.0) then
         goto (851,852,853,854,855,856,857,858),ib
      endif
c
      ic = 0
      if ((ia.eq.1).or.(ia.eq.2)) goto 182
      if ((ia.eq.3).or.(ia.eq.4)) goto 183
      if ((ia.eq.5).or.(ia.eq.6)) goto 184
      if ((ia.eq.7).or.(ia.eq.8)) goto 181
c
c     --- 2 ---
 181  if (ic.ge.3) goto 300
      ic = ic + 1
      if (tri(1,i,j)) goto 182
      z1 = z(i,j)
      z2 = z(i+1,j)
      if (z1.ge.zval) goto 300
      if (z2.eq.zval) goto 853
      if (z2.ge.zval) goto 852
c
c     --- 4 ---
 182  if (ic.ge.3) goto 300
      ic = ic + 1
      if (tri(2,i,j)) goto 183
      z1 = z(i+1,j)
      z2 = z(i+1,j+1)
      if (z1.ge.zval) goto 300
      if (z2.eq.zval) goto 855
      if (z2.ge.zval) goto 854
c
c     --- 6 ---
 183  if (ic.ge.3) goto 300
      ic = ic + 1
      if (tri(3,i,j)) goto 184
      z1 = z(i+1,j+1)
      z2 = z(i,j+1)
      if (z1.ge.zval) goto 300
      if (z2.eq.zval) goto 857
      if (z2.ge.zval) goto 856
c
c     --- 8 ---
 184  if (ic.ge.3) goto 300
      ic = ic + 1
      if (tri(4,i,j)) goto 181
      z1 = z(i,j+1)
      z2 = z(i,j)
      if (z1.ge.zval) goto 300
      if (z2.eq.zval) goto 851
      if (z2.ge.zval) goto 858
      goto 181
c
 751  xx = x(i+1,j)
      yy = y(i+1,j)
      tri(3,i,j-1)=.true.
      in = 7
      i = i+1
      ia = 9
      goto 420
c
 752  x1 = x(i+1,j)
      y1 = y(i+1,j)
      z1 = z(i+1,j)
      x2 = 0.25*(x(i,j)+x(i+1,j)+x(i+1,j+1)+x(i,j+1))
      y2 = 0.25*(y(i,j)+y(i+1,j)+y(i+1,j+1)+y(i,j+1))
      z2 = 0.25*(z(i,j)+z(i+1,j)+z(i+1,j+1)+z(i,j+1))
      tri(1,i,j)=.true.
      in = 4
      goto 400
c
 753  xx = 0.25*(x(i,j)+x(i+1,j)+x(i+1,j+1)+x(i,j+1))
      yy = 0.25*(y(i,j)+y(i+1,j)+y(i+1,j+1)+y(i,j+1))
      tri(1,i,j)=.true.
      in = 8
      ia = 1
      goto 420
c
 754  x1 = 0.25*(x(i,j)+x(i+1,j)+x(i+1,j+1)+x(i,j+1))
      y1 = 0.25*(y(i,j)+y(i+1,j)+y(i+1,j+1)+y(i,j+1))
      z1 = 0.25*(z(i,j)+z(i+1,j)+z(i+1,j+1)+z(i,j+1))
      x2 = x(i,j+1)
      y2 = y(i,j+1)
      z2 = z(i,j+1)
      tri(4,i,j)=.true.
      in = 5
      goto 400
c
 755  xx = x(i,j+1)
      yy = y(i,j+1)
      tri(4,i,j)=.true.
      in = 7
      j = j+1
      ia = 13
      goto 420
c
 756  x1 = x(i,j+1)
      y1 = y(i,j+1)
      z1 = z(i,j+1)
      tri(2,i-1,j)=.true.
      x2 = 0.25*(x(i-1,j)+x(i,j)+x(i,j+1)+x(i-1,j+1))
      y2 = 0.25*(y(i-1,j)+y(i,j)+y(i,j+1)+y(i-1,j+1))
      z2 = 0.25*(z(i-1,j)+z(i,j)+z(i,j+1)+z(i-1,j+1))
      in = 6
      i = i-1
      goto 400
c
 757  xx = 0.25*(x(i-1,j)+x(i,j)+x(i,j+1)+x(i-1,j+1))
      yy = 0.25*(y(i-1,j)+y(i,j)+y(i,j+1)+y(i-1,j+1))
      tri(2,i-1,j)=.true.
      in = 8
      ia = 3
      i = i-1
      goto 420
c
 758  x1 = 0.25*(x(i-1,j)+x(i,j)+x(i,j+1)+x(i-1,j+1))
      y1 = 0.25*(y(i-1,j)+y(i,j)+y(i,j+1)+y(i-1,j+1))
      z1 = 0.25*(z(i-1,j)+z(i,j)+z(i,j+1)+z(i-1,j+1))
      x2 = x(i-1,j)
      y2 = y(i-1,j)
      z2 = z(i-1,j)
      tri(1,i-1,j)=.true.
      in = 3
      i = i-1
      goto 400
c
 759  xx = x(i-1,j)
      yy = y(i-1,j)
      tri(1,i-1,j)=.true.
      in = 7
      i = i-1
      ia = 1
      goto 420
c
 760  x1 = x(i-1,j)
      y1 = y(i-1,j)
      z1 = z(i-1,j)
      x2 = 0.25*(x(i-1,j-1)+x(i,j-1)+x(i,j)+x(i-1,j))
      y2 = 0.25*(y(i-1,j-1)+y(i,j-1)+y(i,j)+y(i-1,j))
      z2 = 0.25*(z(i-1,j-1)+z(i,j-1)+z(i,j)+z(i-1,j))
      tri(3,i-1,j-1)=.true.
      in = 5
      i = i-1
      j = j-1
      goto 400
c
 761  xx = 0.25*(x(i-1,j-1)+x(i,j-1)+x(i,j)+x(i-1,j))
      yy = 0.25*(y(i-1,j-1)+y(i,j-1)+y(i,j)+y(i-1,j))
      tri(3,i-1,j-1)=.true.
      in = 8
      i = i-1
      j = j-1
      ia = 5
      goto 420
c
 762  x1 = 0.25*(x(i-1,j-1)+x(i,j-1)+x(i,j)+x(i-1,j))
      y1 = 0.25*(y(i-1,j-1)+y(i,j-1)+y(i,j)+y(i-1,j))
      z1 = 0.25*(z(i-1,j-1)+z(i,j-1)+z(i,j)+z(i-1,j))
      x2 = x(i,j-1)
      y2 = y(i,j-1)
      z2 = z(i,j-1)
      tri(2,i-1,j-1)=.true.
      in = 4
      i = i-1
      j = j-1
      goto 400
c
 763  xx = x(i,j-1)
      yy = y(i,j-1)
      tri(2,i-1,j-1)=.true.
      in = 7
      j = j-1
      ia = 5
      goto 420
c
 764  x1 = x(i,j-1)
      y1 = y(i,j-1)
      z1 = z(i,j-1)
      x2 = 0.25*(x(i,j-1)+x(i+1,j-1)+x(i+1,j)+x(i,j))
      y2 = 0.25*(y(i,j-1)+y(i+1,j-1)+y(i+1,j)+y(i,j))
      z2 = 0.25*(z(i,j-1)+z(i+1,j-1)+z(i+1,j)+z(i,j))
      tri(4,i,j-1)=.true.
      in = 3
      j = j-1
      goto 400
c
 765  xx = 0.25*(x(i,j-1)+x(i+1,j-1)+x(i+1,j)+x(i,j))
      yy = 0.25*(y(i,j-1)+y(i+1,j-1)+y(i+1,j)+y(i,j))
      tri(4,i,j-1)=.true.
      in = 8
      j = j-1
      ia = 7
      goto 420
c
 766  x1 = 0.25*(x(i,j-1)+x(i+1,j-1)+x(i+1,j)+x(i,j))
      y1 = 0.25*(y(i,j-1)+y(i+1,j-1)+y(i+1,j)+y(i,j))
      z1 = 0.25*(z(i,j-1)+z(i+1,j-1)+z(i+1,j)+z(i,j))
      x2 = x(i+1,j)
      y2 = y(i+1,j)
      z2 = z(i+1,j)
      tri(3,i,j-1)=.true.
      in = 6
      j = j-1
      goto 400
c
 851  xx = x(i,j)
      yy = y(i,j)
      tri(4,i,j)=.true.
      in = 7
      ia = 3
      goto 420 
c
 852  x1 = x(i,j)
      y1 = y(i,j)
      z1 = z(i,j)
      x2 = x(i+1,j)
      y2 = y(i+1,j)
      z2 = z(i+1,j)
      tri(1,i,j)=.true.
      in = 1
      goto 400
c
 853  xx = x(i+1,j)
      yy = y(i+1,j) 
      tri(1,i,j)=.true.
      in = 7
      ia = 7
      i = i+1
      goto 420
c
 854  x1 = x(i+1,j)
      y1 = y(i+1,j)
      z1 = z(i+1,j)
      x2 = x(i+1,j+1)
      y2 = y(i+1,j+1)
      z2 = z(i+1,j+1)
      tri(2,i,j)=.true.
      in = 2
      i = i+1
      goto 400
c
 855  xx = x(i+1,j+1)
      yy = y(i+1,j+1)
      tri(2,i,j)=.true.
      in = 7
      ia = 11
      i = i+1
      j = j+1
      goto 420
c
 856  x1 = x(i+1,j+1)
      y1 = y(i+1,j+1)
      z1 = z(i+1,j+1)
      x2 = x(i,j+1)
      y2 = y(i,j+1)
      z2 = z(i,j+1)
      tri(3,i,j)=.true.
      in =1
      j = j+1
      goto 400
c
 857  xx = x(i,j+1)
      yy = y(i,j+1)
      tri(3,i,j)=.true.
      in = 7
      ia = 15
      j = j+1
      goto 420
c
 858  x1 = x(i,j+1)
      y1 = y(i,j+1)
      z1 = z(i,j+1)
      x2 = x(i,j)
      y2 = y(i,j)
      z2 = z(i,j)
      tri(4,i,j)=.true.
      in =2
      goto 400
c
 400  dz = z2-z1
      if (abs(dz).lt.eps) then
         xx = 0.5*(x1+x2)
         yy = 0.5*(y1+y2)
      else
         dz1 = (zval-z1)/dz
         dz2 = (zval-z2)/dz  
         xx = dz1*x2-dz2*x1
         yy = dz1*y2-dz2*y1
      endif
 420  if ((xx.eq.xx0).and.(yy.eq.yy0)) goto 410
      write(nd,'(2E13.5,3I5)') xx,yy
      xx0=xx
      yy0=yy
 410  goto 100
c
c     --- both sides of the segment are full
c  
 300  write(nd,*) '= ='
c
 1000  return
      end
c
c
c
      integer function nglen(s)
c     *****************************************************************
c     * nglen --- return length of string
c     *   
c     * This function returns 1st posion of ' ' in s.
c     *
c     *    character*80  s    : string
c     *
c     *****************************************************************
      character*80 s
      do 10 i=1,80
         if (s(i:i).eq.' ') goto 20
 10   continue
 20   nglen = i-1
      return
      end
c
c
c
      subroutine ngsti(n,s)
c     *****************************************************************
c     * ngsti --- convert from integer to string
c     *   
c     *    integer       n    : integer number
c     *    character*80  s    : string
c     *
c     *****************************************************************
      character*80 s
      integer n
      integer nn
      character*10 num
      character c
c
      num='0123456789'
      s=' '
      nn=n
      if (nn.lt.0) then
         s='-'
         nn=-nn
      endif
      do 10 idx=5,0,-1
         if (nn.ge.10**idx) goto 20
 10   continue
      s='0'
      goto 40
c
 20   do 30 i=idx,0,-1
         iord=10**i
         nx=nn/iord
         c=num(nx+1:nx+1)
         ip=nglen(s)
         s(ip+1:ip+1)=c
         if (nx.ge.1) nn=nn-nx*iord
 30   continue
c
 40   return
      end
c
c
c
      subroutine ngrngs(nx,ny,x,xmin,imin,jmin,xmax,imax,jmax,
     &     i1,j1,i2,j2,nmsk) 
c     *****************************************************************
c     *  ngrngs - return minimum and maximum values and its indices of
c     *          a array variable, specified area 
c     *
c     *   integer          nx,ny : sizes of arrays
c     *   real          x(nx,ny) : array variable
c     *   real              xmin : minimum value
c     *   integer      imin,jmin : indices of minimum value
c     *   real              xmin : maximum value
c     *   integer      imin,jmin : indices of maximum value
c     *   integer    i1,j1,i2,j2 : range of drowing by grid index
c     *   integer    nmsk(nx,ny) : mask array, 0:draw, else:neglect     
c     *
c     *****************************************************************
      dimension x(nx,ny)
      integer nmsk(nx,ny)
c
c     --- find initial point
      do 20 j=j1,j2
         do 20 i=i1,i2
            if (nmsk(i,j).eq.0) then
               imin=i
               jmin=j
               xmin=x(i,j)
               imax=i
               jmax=j
               xmax=x(i,j)
               goto 30
            endif
 20   continue
c
c     --- get minimum and maximum value
 30   do 10 j=j1,j2
         do 10 i=i1,i2
            if (nmsk(i,j).ne.0) goto 10
            if (x(i,j).gt.xmax) then
               xmax=x(i,j)
               imax=i
               jmax=j
            endif
            if (x(i,j).lt.xmin) then
               xmin=x(i,j)
               imin=i
               jmin=j
            endif
 10   continue
c
      return
      end
c
c
c
      subroutine ngrnga(nx,ny,x,xmin,imin,jmin,xmax,imax,jmax) 
c     *****************************************************************
c     *  ngrnga - return minimum and maximum values and its indices of
c     *          a array variable,all area
c     *
c     *   integer          nx,ny : sizes of arrays
c     *   real          x(nx,ny) : array variable
c     *   real              xmin : minimum value
c     *   integer      imin,jmin : indices of minimum value
c     *   real              xmin : maximum value
c     *   integer      imin,jmin : indices of maximum value
c     *
c     *****************************************************************
      dimension x(nx,ny)
      imin=1
      jmin=1
      xmin=x(1,1)
      imax=1
      jmax=1
      xmax=x(1,1)
      do 10 j=1,ny
         do 10 i=1,nx
            if (x(i,j).gt.xmax) then
               xmax=x(i,j)
               imax=i
               jmax=j
            endif
            if (x(i,j).lt.xmin) then
               xmin=x(i,j)
               imin=i
               jmin=j
            endif
 10   continue
c
      return
      end
c
c
c
      subroutine ngij(nx,ny,x,y,i1,j1,i2,j2,xx,yy,ii,jj) 
c     *****************************************************************
c     * ngij --- return grid index of point (xx,yy)
c     *
c     *    integer          nx,ny : size of arrays
c     *    real x(nx,ny),y(nx,ny) : positions of grid points
c     *    real             xx,yy : cordinate of input point
c     *    integer          ii,jj : result grid index
c     *    
c     *****************************************************************
      dimension x(nx,ny),y(nx,ny)
c
      f(x1,y1,x2,y2,xx,yy)=(yy-y1)*(x2-x1)-(y2-y1)*(xx-x1)
c
      if ((ii.lt.i1).or.(jj.lt.j1)) goto  40
      if ((ii.ge.i2).or.(jj.ge.j2)) goto 40
c
c     --- check current grid
      x1=x(ii,jj+1)
      y1=y(ii,jj+1)
      x2=x(ii,jj)
      y2=y(ii,jj)
      x3=x(ii+1,jj)
      y3=y(ii+1,jj)
      x4=x(ii+1,jj+1)
      y4=y(ii+1,jj+1)
      if (f(x1,y1,x2,y2,xx,yy).lt.0.0) goto 30
      if (f(x2,y2,x3,y3,xx,yy).lt.0.0) goto 30
      if (f(x3,y3,x4,y4,xx,yy).le.0.0) goto 30
      if (f(x4,y4,x1,y1,xx,yy).le.0.0) goto 30
      goto 20
c
c     --- check next grid
 30   i=ii+1
      if (i.ge.i2) goto 31
      j=jj
      x1=x(i,j+1)
      y1=y(i,j+1)
      x2=x(i,j)
      y2=y(i,j)
      x3=x(i+1,j)
      y3=y(i+1,j)
      x4=x(i+1,j+1)
      y4=y(i+1,j+1)
      if (f(x1,y1,x2,y2,xx,yy).lt.0.0) goto 31
      if (f(x2,y2,x3,y3,xx,yy).lt.0.0) goto 31
      if (f(x3,y3,x4,y4,xx,yy).le.0.0) goto 31
      if (f(x4,y4,x1,y1,xx,yy).le.0.0) goto 31
      ii = i
      jj = j
      goto 20
 31   i=ii
      j=jj+1
      if (j.ge.j2) goto 32
      x1=x(i,j+1)
      y1=y(i,j+1)
      x2=x(i,j)
      y2=y(i,j)
      x3=x(i+1,j)
      y3=y(i+1,j)
      x4=x(i+1,j+1)
      y4=y(i+1,j+1)
      if (f(x1,y1,x2,y2,xx,yy).lt.0.0) goto 32
      if (f(x2,y2,x3,y3,xx,yy).lt.0.0) goto 32
      if (f(x3,y3,x4,y4,xx,yy).le.0.0) goto 32
      if (f(x4,y4,x1,y1,xx,yy).le.0.0) goto 32
      ii = i
      jj = j
      goto 20
 32   i=ii-1
      if (i.lt.i1) goto 33 
      j=jj
      x1=x(i,j+1)
      y1=y(i,j+1)
      x2=x(i,j)
      y2=y(i,j)
      x3=x(i+1,j)
      y3=y(i+1,j)
      x4=x(i+1,j+1)
      y4=y(i+1,j+1)
      if (f(x1,y1,x2,y2,xx,yy).lt.0.0) goto 33
      if (f(x2,y2,x3,y3,xx,yy).lt.0.0) goto 33
      if (f(x3,y3,x4,y4,xx,yy).le.0.0) goto 33
      if (f(x4,y4,x1,y1,xx,yy).le.0.0) goto 33
      ii = i
      jj = j
      goto 20
 33   i=ii
      j=jj-1
      if (j.lt.j1) goto 34
      x1=x(i,j+1)
      y1=y(i,j+1)
      x2=x(i,j)
      y2=y(i,j)
      x3=x(i+1,j)
      y3=y(i+1,j)
      x4=x(i+1,j+1)
      y4=y(i+1,j+1)
      if (f(x1,y1,x2,y2,xx,yy).lt.0.0) goto 34
      if (f(x2,y2,x3,y3,xx,yy).lt.0.0) goto 34
      if (f(x3,y3,x4,y4,xx,yy).le.0.0) goto 34
      if (f(x4,y4,x1,y1,xx,yy).le.0.0) goto 34
      ii = i
      jj = j
      goto 20
c
 34   i=ii-1
      j=jj-1
      if (j.lt.j1) goto 35
      if (i.lt.i1) goto 35
      x1=x(i,j+1)
      y1=y(i,j+1)
      x2=x(i,j)
      y2=y(i,j)
      x3=x(i+1,j)
      y3=y(i+1,j)
      x4=x(i+1,j+1)
      y4=y(i+1,j+1)
      if (f(x1,y1,x2,y2,xx,yy).lt.0.0) goto 35
      if (f(x2,y2,x3,y3,xx,yy).lt.0.0) goto 35
      if (f(x3,y3,x4,y4,xx,yy).le.0.0) goto 35
      if (f(x4,y4,x1,y1,xx,yy).le.0.0) goto 35
      ii = i
      jj = j
      goto 20
 35   i=ii+1
      j=jj-1
      if (j.lt.j1) goto 36
      if (i.ge.i2) goto 36
      x1=x(i,j+1)
      y1=y(i,j+1)
      x2=x(i,j)
      y2=y(i,j)
      x3=x(i+1,j)
      y3=y(i+1,j)
      x4=x(i+1,j+1)
      y4=y(i+1,j+1)
      if (f(x1,y1,x2,y2,xx,yy).lt.0.0) goto 36
      if (f(x2,y2,x3,y3,xx,yy).lt.0.0) goto 36
      if (f(x3,y3,x4,y4,xx,yy).le.0.0) goto 36
      if (f(x4,y4,x1,y1,xx,yy).le.0.0) goto 36
      ii = i
      jj = j
      goto 20
 36   i=ii+1
      j=jj+1
      if (j.ge.j2) goto 37
      if (i.ge.i2) goto 37
      x1=x(i,j+1)
      y1=y(i,j+1)
      x2=x(i,j)
      y2=y(i,j)
      x3=x(i+1,j)
      y3=y(i+1,j)
      x4=x(i+1,j+1)
      y4=y(i+1,j+1)
      if (f(x1,y1,x2,y2,xx,yy).lt.0.0) goto 37
      if (f(x2,y2,x3,y3,xx,yy).lt.0.0) goto 37
      if (f(x3,y3,x4,y4,xx,yy).le.0.0) goto 37
      if (f(x4,y4,x1,y1,xx,yy).le.0.0) goto 37
      ii = i
      jj = j
      goto 20
 37   i=ii-1
      j=jj+1
      if (i.lt.i1) goto 40
      if (j.ge.j2) goto 40
      x1=x(i,j+1)
      y1=y(i,j+1)
      x2=x(i,j)
      y2=y(i,j)
      x3=x(i+1,j)
      y3=y(i+1,j)
      x4=x(i+1,j+1)
      y4=y(i+1,j+1)
      if (f(x1,y1,x2,y2,xx,yy).lt.0.0) goto 40
      if (f(x2,y2,x3,y3,xx,yy).lt.0.0) goto 40
      if (f(x3,y3,x4,y4,xx,yy).le.0.0) goto 40
      if (f(x4,y4,x1,y1,xx,yy).le.0.0) goto 40
      ii = i
      jj = j
      goto 20
c
 40   do 10 j=j1,j2-1
         do 10 i=i1,i2-1
            x1=x(i,j+1)
            y1=y(i,j+1)
            x2=x(i,j)
            y2=y(i,j)
            x3=x(i+1,j)
            y3=y(i+1,j)
            x4=x(i+1,j+1)
            y4=y(i+1,j+1)
            if (f(x1,y1,x2,y2,xx,yy).lt.0.0) goto 10
            if (f(x2,y2,x3,y3,xx,yy).lt.0.0) goto 10
            if (f(x3,y3,x4,y4,xx,yy).le.0.0) goto 10
            if (f(x4,y4,x1,y1,xx,yy).le.0.0) goto 10
            ii = i
            jj = j
            goto 20
 10   continue
      ii = -1
      jj = -1  
 20   return
      end
c
c
c
      subroutine ngsrec(nd,fname,zval1,zval2,ir,ig,ib,ileg,ix,iy)
c     *****************************************************************
c     * ngslin  -- output Ngraph script for rectangle solid fill
c     *
c     *   integer         nd : I/O unit
c     *   character*80 fname : name of data file
c     *   real   zval1,zval2 : values indicated as legend
c     *   integer   ir,ig,ib : color (red,green,blue 0:255)
c     *   integer       ileg : 1: with legend, 0: without legend
c     *   integer      ix,iy : position of legend
c     *
c     *****************************************************************
      character*80 dstr,nstr,fname,tilda
c     --- file information
      dstr='new file'
      write(nd,'(A8)') dstr
      dstr= 'file::file=''' // fname(1:nglen(fname)) // ''''
      write(nd,*) dstr(1:nglen(dstr)) 
      dstr= 'file::x=1'
      write(nd,*) dstr(1:nglen(dstr))
      dstr= 'file::y=3'
      write(nd,*) dstr(1:nglen(dstr))
      dstr= 'file::type=rectangle_solid_fill'
      write(nd,*) dstr(1:nglen(dstr))
      call ngsti(ir,dstr)
      dstr = 'file::R=' // dstr
      write(nd,*) dstr(1:nglen(dstr))
      call ngsti(ig,dstr)
      dstr = 'file::G=' // dstr
      write(nd,*) dstr(1:nglen(dstr))
      call ngsti(ib,dstr)
      dstr = 'file::B=' // dstr
      write(nd,*) dstr(1:nglen(dstr))
c     --- legend information
      if (ileg.eq.1) then
c        --- text
         dstr='new text'
         write(nd,'(A8)') dstr
         call ngsti(ix+1000,dstr)
         dstr='text::x=' // dstr
         write(nd,*) dstr(1:nglen(dstr))
         call ngsti(iy,dstr)
         dstr = 'text::y=' // dstr
         write(nd,*) dstr(1:nglen(dstr))
         dstr = 'text::pt=800'
         write(nd,*) dstr(1:nglen(dstr))
         nstr = ' text::text='''
         dstr = ''''
         tilda = ' ~ ' 
         write(nd,'(A13,1pe13.5,A3,1pe13.5,A1)')
     &        nstr,zval1,tilda,zval2,dstr
         call ngsti(ir,dstr)
         dstr = 'text::R=' // dstr
         write(nd,*) dstr(1:nglen(dstr))
         call ngsti(ig,dstr)
         dstr = 'text::G=' // dstr
         write(nd,*) dstr(1:nglen(dstr))
         call ngsti(ib,dstr)
         dstr = 'text::B=' // dstr
         write(nd,*) dstr(1:nglen(dstr))
c        --- rectangle
         dstr='new rectangle'
         write(nd,'(A13)') dstr
         call ngsti(ir,dstr)
         dstr = 'rectangle::R=' // dstr
         write(nd,*) dstr(1:nglen(dstr))
         call ngsti(ig,dstr)
         dstr = 'rectangle::G=' // dstr
         write(nd,*) dstr(1:nglen(dstr))
         call ngsti(ib,dstr)
         dstr = 'rectangle::B=' // dstr
         write(nd,*) dstr(1:nglen(dstr))
         dstr = 'rectangle::fill=true'
         write(nd,*) dstr(1:nglen(dstr))
         ix1=ix
         iy1=iy-300
         ix2=ix+800
         iy2=iy
         call ngsti(ix1,dstr)
         dstr = 'rectangle::x1=' // dstr
         write(nd,*) dstr(1:nglen(dstr))
         call ngsti(iy1,dstr)
         dstr = 'rectangle::y1=' // dstr
         write(nd,*) dstr(1:nglen(dstr))
         call ngsti(ix2,dstr)
         dstr = 'rectangle::x2=' // dstr
         write(nd,*) dstr(1:nglen(dstr))
         call ngsti(iy2,dstr)
         dstr = 'rectangle::y2=' // dstr
         write(nd,*) dstr(1:nglen(dstr))
      endif
      return
      end
c
c
c
      subroutine ngsarw(nd,fac,ix,iy,ref)
c     *****************************************************************
c     *  ngsarw - reference arrow
c     *
c     *   integer         nd : I/O unit
c     *   real           fac : factor of arrow length
c     *   integer      ix,iy : position of legend
c     *   real           ref : magnitude of legend reference vector
c     *
c     *****************************************************************
      character*80 nstr,dstr
      write(nd,'(A19)') 'MIN=${axis:fX1:min}'
      write(nd,'(A19)') 'MAX=${axis:fX1:max}'
      write(nd,'(A19)') 'AXISX=${axis:fX1:x}'
      write(nd,'(A24)') 'AXISL=${axis:fX1:length}'
      call ngsti(ix,dstr)
      dstr='X1=' // dstr
      write(nd,*) dstr(1:nglen(dstr))
      call ngsti(iy,dstr)
      dstr='Y1=' // dstr
      write(nd,*) dstr(1:nglen(dstr))
      dstr='L='
      write(nd,'(A2,1pe11.5)') dstr,fac*ref
      write(nd,'(A37)') 'X2=`expr "$X1+$AXISL/($MAX-$MIN)*$L"`'
      write(nd,'(A8)') 'new line'
      write(nd,*) 'line::points="$X1 $Y1 $X2 $Y1"'
      write(nd,*) 'line::arrow=end'
      write(nd,'(A8)') 'new text'
      nstr = ' text::text='''
      dstr = ''''
      write(nd,'(A13,1pe13.5,A1)') nstr,ref,dstr
      write(nd,*) 'text::x=$X2'
      nn=iy+80
      call ngsti(nn,dstr)
      dstr='text::y=' // dstr
      write(nd,*) dstr(1:nglen(dstr))
      write(nd,*) 'text::pt=800'
      return
      end
c
c
c
      subroutine ngacs(nd,nx,ny,x,y,u,v,w,i1,j1,i2,j2,nmsk,id,jd,
     &     fac,zval1,zval2)
c     *****************************************************************
c     *  ngaw0 - Arrows of level (zval1,zval2) 
c     *
c     *   integer             nd : I/O unit
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real u(nx,ny),v(nx,ny) : component of vectors
c     *   real          w(nx,ny) : value indicated by colors
c     *   integer    i1,j1,i2,j2 : range of drowing by grid index
c     *   integer    nmsk(nx,ny) : mask array, 0:draw, else:neglect     
c     *   integer          id,jd : skip steps of grids
c     *   real               fac : factor of arrow length
c     *   real       zval1,zval2 : number of color level
c     *
c     *****************************************************************
      dimension x(nx,ny),y(nx,ny),u(nx,ny),v(nx,ny),w(nx,ny),nmsk(nx,ny)
      write(nd,'(a8,1pe14.5,a1,1pe14.5)') '# val = ',zval1,'~',zval2 
      write(nd,'(5(a13))') '#    x       ','     y       ',
     &     '     u       ','     v       ','     w       '
      do 10 j=j1,j2,jd
         do 10 i=i1,i2,id
            if (nmsk(i,j).ne.0) goto 10
            if ((w(i,j).ge.zval1).and.(w(i,j).lt.zval2)) then
               write(nd,100) x(i,j),y(i,j),u(i,j),v(i,j),w(i,j),
     &              x(i,j)+fac*u(i,j),y(i,j)+fac*v(i,j)
            endif
 10   continue
 100  format(7(1pe13.5))
      return
      end
c
c
c
      subroutine ngac1(nd,nx,ny,x,y,u,v,w,i1,j1,i2,j2,nmsk,id,jd,
     &     fac,wval)
c     *****************************************************************
c     *  ngaw1 - Arrows of w=wval 
c     *
c     *   integer             nd : I/O unit
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real u(nx,ny),v(nx,ny) : component of vectors
c     *   real          w(nx,ny) : array
c     *   integer    i1,j1,i2,j2 : range of drowing by grid index
c     *   integer    nmsk(nx,ny) : mask array, 0:draw, else:neglect     
c     *   integer          id,jd : skip steps of grids
c     *   real               fac : factor of arrow length
c     *   real              wval : magnitude of legend reference vector
c     *
c     *****************************************************************
      dimension x(nx,ny),y(nx,ny),u(nx,ny),v(nx,ny),w(nx,ny),nmsk(nx,ny)
      do 10 j=j1,j2,jd
         do 10 i=i1,i2,id
            if (nmsk(i,j).ne.0) goto 10
            if (w(i,j).eq.wval) then
               write(nd,100) x(i,j),y(i,j),u(i,j),v(i,j),w(i,j),
     &              x(i,j)+fac*u(i,j),y(i,j)+fac*v(i,j)
            endif
 10   continue
 100  format(7(1pe13.5))
      return
      end
c
c
c
      subroutine ngarws(nd,fname,zval1,zval2,ir,ig,ib,ileg,ix,iy)
c     *****************************************************************
c     * ngarws  -- output Ngraph script for arrow type
c     *
c     *   integer         nd : I/O unit
c     *   character*80 fname : name of data file
c     *   real   zval1,zvql2 : value indicated as legend
c     *   integer   ir,ig,ib : color (red,green,blue 0:255)
c     *   integer       ileg : 1: with legend, 0: without legend
c     *   integer      ix,iy : position of legend
c     *
c     *****************************************************************
      character*80 dstr,nstr,fname,tilda
c     --- file information
      dstr='new file'
      write(nd,'(A8)') dstr
      dstr= 'file::file=''' // fname(1:nglen(fname)) // ''''
      write(nd,*) dstr(1:nglen(dstr)) 
      dstr= 'file::x=1'
      write(nd,*) dstr(1:nglen(dstr))
      dstr= 'file::y=6'
      write(nd,*) dstr(1:nglen(dstr))
      dstr= 'file::type=arrow'
      write(nd,*) dstr(1:nglen(dstr))
      call ngsti(ir,dstr)
      dstr = 'file::R=' // dstr
      write(nd,*) dstr(1:nglen(dstr))
      call ngsti(ig,dstr)
      dstr = 'file::G=' // dstr
      write(nd,*) dstr(1:nglen(dstr))
      call ngsti(ib,dstr)
      dstr = 'file::B=' // dstr
      write(nd,*) dstr(1:nglen(dstr))
c     --- legend information
      if (ileg.eq.1) then
c        --- text
         dstr='new text'
         write(nd,'(A8)') dstr
         call ngsti(ix+1000,dstr)
         dstr='text::x=' // dstr
         write(nd,*) dstr(1:nglen(dstr))
         call ngsti(iy,dstr)
         dstr = 'text::y=' // dstr
         write(nd,*) dstr(1:nglen(dstr))
         dstr = 'text::pt=800'
         write(nd,*) dstr(1:nglen(dstr))
         nstr = ' text::text='''
         dstr = ''''
         tilda = ' ~ ' 
         write(nd,'(A13,1pe13.5,A3,1pe13.5,A1)')
     &        nstr,zval1,tilda,zval2,dstr
         call ngsti(ir,dstr)
         dstr = 'text::R=' // dstr
         write(nd,*) dstr(1:nglen(dstr))
         call ngsti(ig,dstr)
         dstr = 'text::G=' // dstr
         write(nd,*) dstr(1:nglen(dstr))
         call ngsti(ib,dstr)
         dstr = 'text::B=' // dstr
         write(nd,*) dstr(1:nglen(dstr))
c        --- rectangle
         dstr='new rectangle'
         write(nd,'(A13)') dstr
         call ngsti(ir,dstr)
         dstr = 'rectangle::R=' // dstr
         write(nd,*) dstr(1:nglen(dstr))
         call ngsti(ig,dstr)
         dstr = 'rectangle::G=' // dstr
         write(nd,*) dstr(1:nglen(dstr))
         call ngsti(ib,dstr)
         dstr = 'rectangle::B=' // dstr
         write(nd,*) dstr(1:nglen(dstr))
         dstr = 'rectangle::fill=true'
         write(nd,*) dstr(1:nglen(dstr))
         ix1=ix
         iy1=iy-300
         ix2=ix+800
         iy2=iy
         call ngsti(ix1,dstr)
         dstr = 'rectangle::x1=' // dstr
         write(nd,*) dstr(1:nglen(dstr))
         call ngsti(iy1,dstr)
         dstr = 'rectangle::y1=' // dstr
         write(nd,*) dstr(1:nglen(dstr))
         call ngsti(ix2,dstr)
         dstr = 'rectangle::x2=' // dstr
         write(nd,*) dstr(1:nglen(dstr))
         call ngsti(iy2,dstr)
         dstr = 'rectangle::y2=' // dstr
         write(nd,*) dstr(1:nglen(dstr))
      endif
      return
      end
c
c
c
      subroutine ngrida(nd,fname,nx,ny,x,y) 
c     *****************************************************************
c     * ngrida --- generate data files of grid lines, all area 
c     *
c     *    integer             nd : I/O unit number
c     *    character*80     fname : filename 
c     *    integer          nx,ny : size of arrays
c     *    real x(nx,ny),y(nx,ny) : positions of grid points
c     *    
c     *****************************************************************
      dimension x(nx,ny),y(nx,ny)
      character*80 fname, dname
c
      dname=fname(1:nglen(fname)) // '.part' 
      open(nd,file=dname,status='UNKNOWN')
      dname='new file'
      write(nd,60) dname
      dname= 'file::file="' // fname(1:nglen(fname)) // '.dat"' 
      write(nd,70) dname
      dname= 'file::x=1'
      write(nd,60) dname
      dname= 'file::y=2'
      write(nd,60) dname
      dname= 'file::type=line'
      write(nd,70) dname
      close(nd)
 60   format(A20)
 70   format(A80)
c
      dname = fname(1:nglen(fname)) // '.dat' 
      open(nd,file=dname,status='UNKNOWN')
c
      do 10 j=1,ny
         do 20 i=1,nx
            write(nd,30) x(i,j),y(i,j)
 20      continue
         write(nd,*) '= ='
 10   continue
c     
      do 40 i=1,nx
         do 50 j=1,ny
            write(nd,30) x(i,j),y(i,j)
 50      continue
         write(nd,*) '= ='
 40   continue
c
 30   format(1pe14.5,1x,1pe14.5)
c
      return
      end
c
c
c
      subroutine ngcf0s(nd,nx,ny,x,y,z,i1,j1,i2,j2,nmsk,zval1,zval2,
     &     ires,jres)
c     *****************************************************************
c     * ngcf0  -- fill between levels (zval1,zval2)
c     *
c     *   integer             nd : I/O unit
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real          z(nx,ny) : values of grid points
c     *   integer    i1,j1,i2,j2 : range of drowing area (grid indices)
c     *   integer    nmsk(nx,ny) : mask array, 0:draw, else:neglect     
c     *   real       zval1,zval2 : range of contour level
c     *   integer      ires,jres : resolutions in x,y-directions    
c     *
c     *****************************************************************
      dimension x(nx,ny),y(nx,ny),z(nx,ny)
      integer nmsk(nx,ny)
c
      write(nd,'(a8,1pe16.5,a4,1pe16.5)') '# val = ',zval1,' ~ ',zval2
      call ngrngs(nx,ny,x,xmin,imin,jmin,xmax,imax,jmax,
     &     i1,j1,i2,j2,nmsk)
      call ngrngs(nx,ny,y,ymin,imin,jmin,ymax,imax,jmax,
     &     i1,j1,i2,j2,nmsk)
c
      axy=float((i2-i1)*(j2-j1))
      eps=sqrt((xmax-xmin)*(ymax-ymin))/axy/axy
c
      dx=(xmax-xmin)/float(ires)
      dy=(ymax-ymin)/float(jres)
      dx2=0.5*dx
      dy2=0.5*dy
      iflag = 0
      do 10 j=1,jres
         do 20 i=1,ires
            x0=xmin+dx*float(i-1)
            y0=ymin+dy*float(j-1)
            x1=x0+dx
            y1=y0+dy
            xx=x0+dx2
            yy=y0+dy2
            call ngvals(nx,ny,x,y,z,xx,yy,zz,ii,jj,eps,
     &           i1,j1,i2,j2,nmsk)
c
            if (iflag.eq.0) then
               if   ((((ii.ge.i1).and.(ii.le.i2)).and.
     &              ((jj.ge.j1).and.(jj.le.j2))).and.
     &              ((zz.ge.zval1).and.(zz.lt.zval2))) then
                  iflag = 1
                  xleft = x0
                  yleft = y0
               else
                  goto 20
               endif
            else 
               if   ((((ii.ge.i1).and.(ii.le.i2)).and.
     &              ((jj.ge.j1).and.(jj.le.j2))).and.
     &              ((zz.ge.zval1).and.(zz.lt.zval2))) then
                  goto 20
               else
                  iflag=0
                  write(nd,100) xleft,yleft,x1-dx,y1
               endif
            endif
 20      continue
         if (iflag.eq.1) then
            iflag=0
            write(nd,100) xleft,yleft,x1-dx,y1
         endif
 10   continue
c
 100  format(4(1pe14.5))
c
      return
      end
c
c
c
      subroutine ngc0a(nd,nx,ny,x,y,z,zval)
c     *****************************************************************
c     * ngcl0  -- contour line of level zval
c     *
c     *   integer             nd : I/O unit
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real          z(nx,ny) : values of grid points
c     *   real              zval : contour level     
c     *
c     *****************************************************************
      include 'ng2d_par.f'
      dimension x(nx,ny),y(nx,ny),z(nx,ny)
      logical*1 tri(4,0:mx,0:my)
c
      write(nd,'(a10,1pe16.5)') '# value = ',zval
c
      call ngrnga(nx,ny,z,zmin,imin,jmin,zmax,imax,jmax)
      eps=(zmax-zmin)*1.0e-30
c
      do 20 j=0,ny
         do 20 i=0,nx
            do 20 is=1,4
               tri(is,i,j)=.false.
 20   continue
c
      do 30 i=0,ny
         do 30 is=1,4
            tri(is,i,0)=.true.
            tri(is,i,ny)=.true.
 30   continue
c
      do 40 j=0,nx
         do 40 is=1,4
            tri(is,0,j)=.true.
            tri(is,nx,j)=.true.
 40   continue
c
c      do 65 j=1,ny-1
c         do 65 i=1,nx-1
c            z1=z(i,j)-z(i+1,j)
c            z2=z(i,j)-z(i+1,j+1)
c            z3=z(i,j)-z(i,j+1)
c            if ((z1*z1+z2*z2+z3*z3).eq.0.0) then
c               do 66 is=1,4
c                  tri(is,i,j)=.true.
c 66            continue
c            endif
c 65   continue
c
      do 100 js=1,ny-1
         call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,tri,zval,
     &        7,1,js,eps)
         call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,tri,zval,
     &        2,1,js,eps)
 100  continue
c
      do 105 is=1,nx-1
         call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,tri,zval,
     &        7,is,ny,eps)
         call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,tri,zval,
     &        1,is,ny,eps)
 105  continue
c
c
      do 110 js=ny,2,-1
         call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,tri,zval,
     &        2,nx,js,eps)
         call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,tri,zval,
     &        7,nx,js-1,eps)
  110  continue
c
      do 115 is=nx,2,-1
         call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,tri,zval,
     &        1,i,1,eps)
         call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,tri,zval,
     &        7,i-1,1,eps)
 115  continue
c 
c     ---- inside
      do 130 js=1,ny-1
         do 130 is=1,nx-1
            do 130 ins=1,6
               call ngtcl(nd,nx,ny,x,y,z,1,1,nx,ny,
     &              mx,my,tri,zval,ins,is,js,eps)
 130  continue
c
      return
      end
c
c
c
      subroutine ngc0s(nd,nx,ny,x,y,z,i1,j1,i2,j2,nmsk,zval)
c     *****************************************************************
c     * ngcl0  -- contour line of level zval
c     *
c     *   integer             nd : I/O unit
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real          z(nx,ny) : values of grid points
c     *   integer    i1,j1,i2,j2 : range of drowing area (grid indices)
c     *   integer    nmsk(nx,ny) : mask array, 0:draw, else:neglect     
c     *   real              zval : contour level     
c     *
c     *****************************************************************
      include 'ng2d_par.f'
      dimension x(nx,ny),y(nx,ny),z(nx,ny)
      integer nmsk(nx,ny),iret
      logical*1 tri(4,0:mx,0:my),seg(2,0:mx,0:my)
c
      write(nd,'(a10,1pe16.5)') '# value = ',zval
c
      call ngrngs(nx,ny,z,zmin,imin,jmin,zmax,imax,jmax,
     &     i1,j1,i2,j2,nmsk) 
      eps=(zmax-zmin)*1.0e-30
c
c     --- clear triangle flags
c
      do 20 j=j1-1,j2
         do 20 i=i1-1,i2
            do 20 is=1,4
               tri(is,i,j)=.false.
 20   continue
c
      do 30 i=i1-1,i2
         do 30 is=1,4
            tri(is,i,j1-1)=.true.
            tri(is,i,j2)=.true.
 30   continue
c
      do 40 j=j1-1,j2
         do 40 is=1,4
            tri(is,i1-1,j)=.true.
            tri(is,i2,j)=.true.
 40   continue
c
      do 50 j=j1,j2
         do 50 i=i1,i2
            if (nmsk(i,j).ne.0) then
               do 60 is=1,4
                  tri(is,i,j)=.true.
                  if (i.gt.1) then
                     tri(is,i-1,j)=.true.
                     if (j.gt.1) then
                        tri(is,i-1,j-1)=.true.
                     endif
                  endif
                  if (j.gt.1) then
                     tri(is,i,j-1)=.true.
                  endif
 60            continue
            endif
 50   continue
c
c     --- find start point and trace contour lines
c
c     *** start from external boundary *** 
c
      do 100 js=j1,j2-1
            call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,tri,zval,
     &           7,i1,js+1,eps)
            call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,tri,zval,
     &           7,i1,js+1,eps)
            call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,tri,zval,
     &           2,i1,js,eps)
 100  continue
c
      do 105 is=i1,i2-1
            call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,tri,zval,
     &           7,is+1,j2,eps)
            call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,tri,zval,
     &           7,is+1,j2,eps)
            call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,tri,zval,
     &           1,is,j2,eps)
 105  continue
c
      do 110 js=j2-1,j1,-1
            call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,tri,zval,
     &           7,i2,js,eps)
            call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,tri,zval,
     &           7,i2,js,eps)
            call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,tri,zval,
     &           2,i2,js,eps)
 110  continue
c
      do 115 is=i2-1,i1,-1
            call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,tri,zval,
     &           7,is,j1,eps)
            call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,tri,zval,
     &           7,is,j1,eps)
            call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,tri,zval,
     &           1,is,j1,eps)
 115  continue
c
c     *** start along masking area ***
c
c     --- clear segment flag 
      do 120 j=j1-1,j2
         do 120 i=i1-1,i2
            seg(1,i,j)=.false.
            seg(2,i,j)=.false.
 120  continue
c
c     --- set segment on frame true
      do 121 j=j1-1,j2
         seg(1,i1-1,j)=.true.
         seg(2,i1,j)=.true.
         seg(1,i2,j)=.true.
         seg(2,i2,j)=.true.
 121  continue
      do 122 i=i1-1,i2
         seg(1,i,j1)=.true.
         seg(2,i,j1-1)=.true.
         seg(1,i,j2)=.true.
         seg(2,i,j2)=.true.
 122  continue
c
c     -- left
      do 200 js=j1+1,j2-1
         is = i1
         if (nmsk(is,js).ne.0) goto 200
         i = is
         j = js
         n = 1
 230     i0 = i
         j0 = j
         assign 210 to iret
         goto 2000
 210     if ((i.eq.i0).and.(j.eq.j0)) goto 200
         if (i.eq.i0+1) call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,
     &        tri,zval,1,i0,j0,eps)
         if (j.eq.j0+1) call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,
     &        tri,zval,2,i0,j0,eps)
         if (i.eq.i0-1) call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,
     &        tri,zval,1,i,j,eps)
         if (j.eq.j0-1) call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,
     &        tri,zval,2,i,j,eps)
         do 215 nn=1,4 
            call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,tri,zval,
     &           7,i,j,eps)
 215     continue
         if ((i.eq.is).and.(j.eq.js)) goto 200
         if (i.eq.i1) goto 200
         if (i.eq.i2) goto 200
         if (j.eq.j1) goto 200
         if (j.eq.j2) goto 200
         goto 230
 200  continue
c
c     -- top
      do 300 is = i1+1,i2-1
         js = j2
         if (nmsk(is,js).ne.0) goto 300
         i = is
         j = js
         n = 4
 330     i0 = i
         j0 = j
         assign 310 to iret
         goto 2000
 310     if ((i.eq.i0).and.(j.eq.j0)) goto 300
         if (i.eq.i0+1) call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,
     &        tri,zval,1,i0,j0,eps)
         if (j.eq.j0+1) call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,
     &        tri,zval,2,i0,j0,eps)
         if (i.eq.i0-1) call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,
     &        tri,zval,1,i,j,eps)
         if (j.eq.j0-1) call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,
     &        tri,zval,2,i,j,eps)
         do 315 nn=1,4
            call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,tri,zval,
     &           7,i,j,eps)
 315     continue
         if ((i.eq.is).and.(j.eq.js)) goto 300
         if (i.eq.i1) goto 300
         if (i.eq.i2) goto 300
         if (j.eq.j1) goto 300
         if (j.eq.j2) goto 300
         goto 330
  300  continue
c     -- right
      do 400 js = j2-1,j1+1,-1
         is = i2
         if (nmsk(is,js).ne.0) goto 400
         i = is
         j = js
         n = 3
 430     i0 = i
         j0 = j
         assign 410 to iret
         goto 2000
 410     if ((i.eq.i0).and.(j.eq.j0)) goto 400
         if (i.eq.i0+1) call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,
     &        tri,zval,1,i0,j0,eps)
         if (j.eq.j0+1) call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,
     &        tri,zval,2,i0,j0,eps)
         if (i.eq.i0-1) call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,
     &        tri,zval,1,i,j,eps)
         if (j.eq.j0-1) call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,
     &        tri,zval,2,i,j,eps)
         do 415 nn=1,4
            call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,tri,zval,
     &           7,i,j,eps)
 415     continue
         if ((i.eq.is).and.(j.eq.js)) goto 400
         if (i.eq.i1) goto 400
         if (i.eq.i2) goto 400
         if (j.eq.j1) goto 400
         if (j.eq.j2) goto 400
         goto 430
 400  continue
c     -- bottom
      do 500 is = i2-1,i1+1,-1
         js = j1
         if (nmsk(is,js).ne.0) goto 500
         i = is
         j = js
         n = 2
 530     i0 = i
         j0 = j
         assign 510 to iret
         goto 2000
 510     if ((i.eq.i0).and.(j.eq.j0)) goto 500
         if (i.eq.i0+1) call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,
     &        tri,zval,1,i0,j0,eps)
         if (j.eq.j0+1) call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,
     &        tri,zval,2,i0,j0,eps)
         if (i.eq.i0-1) call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,
     &        tri,zval,1,i,j,eps)
         if (j.eq.j0-1) call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,
     &        tri,zval,2,i,j,eps)
         do 515 nn=1,4
            call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,tri,zval,
     &           7,i,j,eps)
 515     continue
         if ((i.eq.is).and.(j.eq.js)) goto 500
         if (i.eq.i1) goto 500
         if (i.eq.i2) goto 500
         if (j.eq.j1) goto 500
         if (j.eq.j2) goto 500
         goto 530
 500  continue
c
      do 600 js = j1+1,j2-1
         do 600 is = i1+1,i2-1
            i = is
            j = js
            n = 1
 630        i0 = i
            j0 = j
            assign 610 to iret
 610        if ((i.eq.i0).and.(j.eq.j0)) goto 600
            if (i.eq.(i0+1)) call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,
     &           mx,my,tri,zval,1,i0,j0,eps)
            if (j.eq.(j0+1)) call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,
     &           mx,my,tri,zval,2,i0,j0,eps)
            if (i.eq.(i0-1)) call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,
     &           mx,my,tri,zval,1,i,j,eps)
            if (j.eq.(j0-1)) call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,
     &           mx,my,tri,zval,2,i,j,eps)
            do 615 nn=1,4
               call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,mx,my,tri,zval,
     &              7,i,j,eps)
 615        continue
            if ((i.eq.is).and.(j.eq.js)) goto 600
            if (i.eq.i1) goto 600
            if (i.eq.i2) goto 600
            if (j.eq.j1) goto 600
            if (j.eq.j2) goto 600
            goto 630
 600  continue
c
c     ---- inside
      do 130 js=j1,j2-1
         do 130 is=i1,i2-1
            do 131 ins=1,6
               call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,
     &              mx,my,tri,zval,ins,is,js,eps)
 131        continue
            do 132 nn=1,4
               call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,
     &              mx,my,tri,zval,7,is,js,eps)
 132        continue
c     
               call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,
     &              mx,my,tri,zval,8,is,js,eps)
               call ngtcl(nd,nx,ny,x,y,z,i1,j1,i2,j2,
     &              mx,my,tri,zval,8,is,js,eps)
 130  continue
c
      goto 3000
c
c     --- local subroutine for find next point
 2000 ic = 0
      goto (2010,2020,2030,2040),n
c
 2010 if (ic.ge.4) goto iret
      ic = ic +1
      if (seg(1,i,j)) goto 2020
      if (nmsk(i+1,j).ne.0) goto 2020
      if ((nmsk(i+1,j+1).eq.0).and.(nmsk(i,j+1).eq.0)) goto 2020
      seg(1,i,j)=.true.
      n = 4
      i = i+1
      goto iret
 2020 if (ic.ge.4) goto iret
      ic = ic +1
      if (seg(2,i,j)) goto 2030
      if (nmsk(i,j+1).ne.0) goto 2030
      if ((nmsk(i-1,j+1).eq.0).and.(nmsk(i-1,j).eq.0)) goto 2030
      seg(2,i,j)=.true.
      n = 1
      j = j+1
      goto iret
 2030 if (ic.ge.4) goto iret
      ic = ic +1
      if (seg(1,i-1,j)) goto 2040
      if (nmsk(i-1,j).ne.0) goto 2040
      if ((nmsk(i-1,j-1).eq.0).and.(nmsk(i,j-1).eq.0)) goto 2040
      seg(1,i-1,j)=.true.
      n = 2
      i = i-1
      goto iret
 2040 if (ic.ge.4) goto iret
      ic = ic +1
      if (seg(2,i,j-1)) goto 2010
      if (nmsk(i,j-1).ne.0) goto 2010
      if ((nmsk(i+1,j-1).eq.0).and.(nmsk(i+1,j).eq.0)) goto 2010
      seg(2,i,j-1)=.true.
      n = 3
      j = j-1
      goto iret
c
 3000 return
      end
c
c
c
      subroutine ngcf0a(nd,nx,ny,x,y,z,zval1,zval2,ires,jres)
c     *****************************************************************
c     * ngcf0a  -- fill between levels (zval1,zval2), all area
c     *
c     *   integer             nd : I/O unit
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real          z(nx,ny) : values of grid points
c     *   real       zval1,zval2 : range of contour level
c     *   integer      ires,jres : resolutions in x,y-directions    
c     *
c     *****************************************************************
      dimension x(nx,ny),y(nx,ny),z(nx,ny)
c
      write(nd,'(a10,1pe16.5,a3,1pe16.5)')
     &     '# value = ',zval1,' ~ ',zval2
      call ngrnga(nx,ny,x,xmin,imin,jmin,xmax,imax,jmax)
      call ngrnga(nx,ny,y,ymin,imin,jmin,ymax,imax,jmax)
c
      axy=float((nx-1)*(ny-1))
      eps=sqrt((xmax-xmin)*(ymax-ymin))/axy/axy
c
      dx=(xmax-xmin)/float(ires)
      dy=(ymax-ymin)/float(jres)
      dx2=0.5*dx
      dy2=0.5*dy
      iflag = 0
      do 10 j=1,jres
         do 20 i=1,ires
            x0=xmin+dx*float(i-1)
            y0=ymin+dy*float(j-1)
            x1=x0+dx
            y1=y0+dy
            xx=x0+dx2
            yy=y0+dy2
            call ngvala(nx,ny,x,y,z,xx,yy,zz,ii,jj,eps)
c
            if (iflag.eq.0) then
               if   ((((ii.ge.1).and.(ii.le.nx)).and.
     &              ((jj.ge.1).and.(jj.le.ny))).and.
     &              ((zz.ge.zval1).and.(zz.lt.zval2))) then
                  iflag = 1
                  xleft = x0
                  yleft = y0
               else
                  goto 20
               endif
            else 
               if   ((((ii.ge.1).and.(ii.le.nx)).and.
     &              ((jj.ge.1).and.(jj.le.ny))).and.
     &              ((zz.ge.zval1).and.(zz.lt.zval2))) then
                  goto 20
               else
                  iflag=0
                  write(nd,100) xleft,yleft,x1-dx,y1
               endif
            endif
 20      continue
         if (iflag.eq.1) then
            iflag=0
            write(nd,100) xleft,yleft,x1-dx,y1
         endif
 10   continue
c
 100  format(4(1pe14.5))
c
      return
      end
c
c
c
      subroutine ngaca(nd,nx,ny,x,y,u,v,w,id,jd,fac,zval1,zval2)
c     *****************************************************************
c     *  ngaw0 - Arrows of level (zval1,zval2) 
c     *
c     *   integer             nd : I/O unit
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real u(nx,ny),v(nx,ny) : component of vectors
c     *   real          w(nx,ny) : value indicated by colors
c     *   integer          id,jd : skip steps of grids
c     *   real               fac : factor of arrow length
c     *   real       zval1,zval2 : number of color level
c     *
c     *****************************************************************
      dimension x(nx,ny),y(nx,ny),u(nx,ny),v(nx,ny),w(nx,ny)
c
      write(nd,'(a8,1pe14.5,a1,1pe14.5)') '# val = ',zval1,'~',zval2 
      write(nd,'(5(a13))') '#    x       ','     y       ',
     &     '     u       ','     v       ','     w       '
      i1 = id/2
      j1 = jd/2
c
      do 10 j=j1,ny,jd
         do 10 i=i1,nx,id
            if ((w(i,j).ge.zval1).and.(w(i,j).lt.zval2)) then
               write(nd,100) x(i,j),y(i,j),u(i,j),v(i,j),w(i,j),
     &              x(i,j)+fac*u(i,j),y(i,j)+fac*v(i,j)
            endif
 10   continue

 100  format(7(1pe13.5))
      return
      end
c
c
c
      subroutine ngac2(nd,nx,ny,x,y,u,v,w,id,jd,
     &     fac,wval)
c     *****************************************************************
c     *  ngaw1 - Arrows of level w=wval 
c     *
c     *   integer             nd : I/O unit
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real u(nx,ny),v(nx,ny) : component of vectors
c     *   real          w(nx,ny) : array
c     *   integer          id,jd : skip steps of grids
c     *   real               fac : factor of arrow length
c     *   real              wval : magnitude of legend reference vector
c     *
c     *****************************************************************
      dimension x(nx,ny),y(nx,ny),u(nx,ny),v(nx,ny),w(nx,ny)
      i1=id/2
      j1=jd/2
      do 10 j=j1,ny,jd
         do 10 i=i1,nx,id
            if (w(i,j).eq.wval) then
               write(nd,100) x(i,j),y(i,j),u(i,j),v(i,j),w(i,j),
     &              x(i,j)+fac*u(i,j),y(i,j)+fac*v(i,j)
            endif
 10   continue
 100  format(7(1pe13.5))
      return
      end
c
c
c
      subroutine ngrids(nd,fname,nx,ny,x,y,i1,j1,i2,j2,nmsk) 
c     *****************************************************************
c     * nggdln --- generate data files of grid lines 
c     *
c     *   integer             nd : I/O unit number
c     *   character*80     fname : filename 
c     *   integer          nx,ny : size of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   integer    i1,i2,j1,j2 : range of drowing by grid index
c     *   integer    nmsk(nx,ny) : mask array, 0:draw, else:neglect     
c     *    
c     *****************************************************************
      dimension x(nx,ny),y(nx,ny)
      integer nmsk(nx,ny)
      character*80 fname, dname
c
      dname=fname(1:nglen(fname)) // '.part' 
      open(nd,file=dname,status='UNKNOWN')
      dname='new file'
      write(nd,60) dname
      dname= 'file::file="' // fname(1:nglen(fname)) // '.dat"' 
      write(nd,70) dname
      dname= 'file::x=1'
      write(nd,60) dname
      dname= 'file::y=2'
      write(nd,60) dname
      dname= 'file::type=line'
      write(nd,70) dname
      close(nd)
 60   format(A20)
 70   format(A80)
c
      dname = fname(1:nglen(fname)) // '.dat' 
      open(nd,file=dname,status='UNKNOWN')
c
      do 10 j=j1,j2
         do 20 i=i1,i2
            if (nmsk(i,j).ne.0) then
               write(nd,*) '= ='
            else
               write(nd,30) x(i,j),y(i,j)
            endif
 20      continue
         write(nd,*) '= ='
 10   continue
c     
      do 40 i=i1,i2
         do 50 j=j1,j2
            if (nmsk(i,j).ne.0) then
               write(nd,*) '= ='
            else
               write(nd,30) x(i,j),y(i,j)
            endif
 50      continue
         write(nd,*) '= ='
 40   continue
c
 30   format(1pe14.5,1x,1pe14.5)
c
      return
      end
c
c
c
      subroutine ngfrma(nd,fname,nx,ny,x,y) 
c     *****************************************************************
c     * ngfrma --- generate data files of frame lines 
c     *
c     *    integer          nd    : I/O unit number
c     *    character*80     fname : filename 
c     *    integer          nx,ny : size of arrays
c     *    real x(nx,ny),y(nx,ny) : positions of grid points
c     *    
c     *****************************************************************
      dimension x(nx,ny),y(nx,ny)
      character*80 fname, dname
c
      dname=fname(1:nglen(fname)) // '.part' 
      open(nd,file=dname,status='UNKNOWN')
      dname='new file'
      write(nd,60) dname
      dname= 'file::file="' // fname(1:nglen(fname)) // '.dat"' 
      write(nd,70) dname
      dname= 'file::x=1'
      write(nd,60) dname
      dname= 'file::y=2'
      write(nd,60) dname
      dname= 'file::type=line'
      write(nd,70) dname
      close(nd)
 60   format(A20)
 70   format(A80)
c
      dname = fname(1:nglen(fname)) // '.dat' 
      open(nd,file=dname,status='UNKNOWN')
c
      do 10 i=1,nx
         write(nd,30) x(i,1),y(i,1)
 10   continue
c
      do 20 j=2,ny
         write(nd,30) x(nx,j),y(nx,j)
 20   continue
c
      do 40 i=nx-1,1,-1
         write(nd,30) x(i,ny),y(i,ny)
 40   continue
c
      do 50 j=ny-1,1,-1
         write(nd,30) x(1,j),y(1,j) 
 50   continue
c
      close(nd)
c
 30   format(1pe14.5,1x,1pe14.5)
c
      return
      end
c
c
c
      subroutine ngfrms(nd,fname,nx,ny,x,y,i1,j1,i2,j2,nmsk) 
c     *****************************************************************
c     * ngfrms --- generate data files of frame lines 
c     *
c     *    integer            nd  : I/O unit number
c     *    character*80    fname  : filename 
c     *    integer         nx,ny  : size of arrays
c     *    real x(nx,ny),y(nx,ny) : positions of grid points
c     *   integer    i1,i2,j1,j2  : range of drowing by grid index
c     *   integer     nmsk(nx,ny) : mask array, 0:draw, else:neglect     
c     *    
c     *****************************************************************
      include 'ng2d_par.f'
      dimension x(nx,ny),y(nx,ny)
      integer nmsk(nx,ny),iret
      character*80 fname, dname
      logical*1 seg(2,0:mx,0:my),lflag
c
      dname=fname(1:nglen(fname)) // '.part' 
      open(nd,file=dname,status='UNKNOWN')
      dname='new file'
      write(nd,60) dname
      dname= 'file::file="' // fname(1:nglen(fname)) // '.dat"' 
      write(nd,70) dname
      dname= 'file::x=1'
      write(nd,60) dname
      dname= 'file::y=2'
      write(nd,60) dname
      dname= 'file::type=line'
      write(nd,70) dname
      close(nd)
 60   format(A20)
 70   format(A80)
c
      dname = fname(1:nglen(fname)) // '.dat' 
      open(nd,file=dname,status='UNKNOWN')
c
c     --- write external frame ---
      do 10 j=j1,j2
         write(nd,1000) x(i1,j),y(i1,j)
 10   continue
      do 20 i=i1+1,i2
         write(nd,1000) x(i,j2),y(i,j2)
 20   continue
      do 30 j=j2-1,j1,-1
         write(nd,1000) x(i2,j),y(i2,j)
 30   continue
      do 40 i=i2-1,i1,-1
         write(nd,1000) x(i,j1),y(i,j1) 
 40   continue
      write(nd,*) '= ='
c
c     --- clear segment flag 
      do 100 j=j1-1,j2
         do 100 i=i1-1,i2
            seg(1,i,j)=.false.
            seg(2,i,j)=.false.
 100  continue
c
c     --- set segment on frame true
      do 110 j=j1-1,j2
         seg(1,i1-1,j)=.true.
         seg(2,i1,j)=.true.
         seg(1,i2,j)=.true.
         seg(2,i2,j)=.true.
 110  continue
c
      do 120 i=i1-1,i2
         seg(1,i,j1)=.true.
         seg(2,i,j1-1)=.true.
         seg(1,i,j2)=.true.
         seg(2,i,j2)=.true.
 120  continue
c
c     -- left
      do 200 js=j1+1,j2-1
         is = i1
         if (nmsk(is,js).ne.0) goto 200
         i = is
         j = js
         n = 1
         lflag = .true.
 230     i0 = i
         j0 = j
         assign 210 to iret
         goto 2000
 210     if ((i.eq.i0).and.(j.eq.j0)) goto 240
         if ((i.eq.is).and.(j.eq.js)) goto 220
         if (i.eq.i1) goto 220
         if (i.eq.i2) goto 220
         if (j.eq.j1) goto 220
         if (j.eq.j2) goto 220
         if (lflag) then
            write(nd,1000) x(is,js),y(is,js)
            lflag = .false.
         endif
         write(nd,1000) x(i,j),y(i,j)
         goto 230
 220     write(nd,1000) x(i,j),y(i,j)
 240     if (lflag) goto 200
         write(nd,*) '= ='
         lflag = .true.
 200  continue
c
c     -- top
      do 300 is = i1+1,i2-1
         js = j2
         if (nmsk(is,js).ne.0) goto 300
         i = is
         j = js
         n = 4
         lflag = .true.
 330     i0 = i
         j0 = j
         assign 310 to iret
         goto 2000
 310     if ((i.eq.i0).and.(j.eq.j0)) goto 340
         if ((i.eq.is).and.(j.eq.js)) goto 320
         if (i.eq.i1) goto 320
         if (i.eq.i2) goto 320
         if (j.eq.j1) goto 320
         if (j.eq.j2) goto 320
         if (lflag) then
            write(nd,1000) x(is,js),y(is,js)
            lflag = .false.
         endif
         write(nd,1000) x(i,j),y(i,j)
         goto 330
 320     write(nd,1000) x(i,j),y(i,j)
 340     if (lflag) goto 300
         write(nd,*) '= ='
         lflag = .true.
 300  continue
c
c     -- right
      do 400 js = j2-1,j1+1,-1
         is = i2
         if (nmsk(is,js).ne.0) goto 400
         i = is
         j = js
         n = 3
         lflag = .true.
 430     i0 = i
         j0 = j
         assign 410 to iret
         goto 2000
 410     if ((i.eq.i0).and.(j.eq.j0)) goto 440
         if ((i.eq.is).and.(j.eq.js)) goto 420
         if (i.eq.i1) goto 420
         if (i.eq.i2) goto 420
         if (j.eq.j1) goto 420
         if (j.eq.j2) goto 420
         if (lflag) then
            write(nd,1000) x(is,js),y(is,js)
            lflag = .false.
         endif
         write(nd,1000) x(i,j),y(i,j)
         goto 430
 420     write(nd,1000) x(i,j),y(i,j)
 440     if (lflag) goto 400
         write(nd,*) '= ='
         lflag = .true.
 400  continue
c
c     -- bottom
      do 500 is = i2-1,i1+1,-1
         js = j1
         if (nmsk(is,js).ne.0) goto 500
         i = is
         j = js
         n = 2
         lflag = .true.
 530     i0 = i
         j0 = j
         assign 510 to iret
         goto 2000
 510     if ((i.eq.i0).and.(j.eq.j0)) goto 540
         if ((i.eq.is).and.(j.eq.js)) goto 520
         if (i.eq.i1) goto 520
         if (i.eq.i2) goto 520
         if (j.eq.j1) goto 520
         if (j.eq.j2) goto 520
         if (lflag) then
            write(nd,1000) x(is,js),y(is,js)
            lflag = .false.
         endif
         write(nd,1000) x(i,j),y(i,j)
         goto 530
 520     write(nd,1000) x(i,j),y(i,j)
 540     if (lflag) goto 500
         write(nd,*) '= ='
         lflag = .true.
 500  continue
c
      do 600 js = j1+1,j2-1
         do 600 is = i1+1,i2-1
            if (nmsk(is,js).ne.0) goto 600
            i = is
            j = js
            n = 1
            lflag = .true.
 630        i0 = i
            j0 = j
            assign 610 to iret
            goto 2000
 610        if ((i.eq.i0).and.(j.eq.j0)) goto 640
            if ((i.eq.is).and.(j.eq.js)) goto 620
            if (i.eq.i1) goto 620
            if (i.eq.i2) goto 620
            if (j.eq.j1) goto 620
            if (j.eq.j2) goto 620
            if (lflag) then
               write(nd,1000) x(is,js),y(is,js)
               lflag = .false.
            endif
            write(nd,1000) x(i,j),y(i,j)
            goto 630
 620        write(nd,1000) x(i,j),y(i,j)
 640        if (lflag) goto 600
            write(nd,*) '= ='
            lflag = .true.
 600  continue
c
      close(nd)
c
      goto 3000
c
 1000 format(1pe14.5,1x,1pe14.5)
c
c     --- local subroutine for find next point
 2000 ic = 0
      goto (2010,2020,2030,2040),n
c
 2010 if (ic.ge.4) goto iret
      ic = ic +1
      if (seg(1,i,j)) goto 2020
      if (nmsk(i+1,j).ne.0) goto 2020
      if ((nmsk(i+1,j+1).eq.0).and.(nmsk(i,j+1).eq.0)) goto 2020
      seg(1,i,j)=.true.
      n = 4
      i = i+1
      goto iret
 2020 if (ic.ge.4) goto iret
      ic = ic +1
      if (seg(2,i,j)) goto 2030
      if (nmsk(i,j+1).ne.0) goto 2030
      if ((nmsk(i-1,j+1).eq.0).and.(nmsk(i-1,j).eq.0)) goto 2030
      seg(2,i,j)=.true.
      n = 1
      j = j+1
      goto iret
 2030 if (ic.ge.4) goto iret
      ic = ic +1
      if (seg(1,i-1,j)) goto 2040
      if (nmsk(i-1,j).ne.0) goto 2040
      if ((nmsk(i-1,j-1).eq.0).and.(nmsk(i,j-1).eq.0)) goto 2040
      seg(1,i-1,j)=.true.
      n = 2
      i = i-1
      goto iret
 2040 if (ic.ge.4) goto iret
      ic = ic +1
      if (seg(2,i,j-1)) goto 2010
      if (nmsk(i,j-1).ne.0) goto 2010
      if ((nmsk(i+1,j-1).eq.0).and.(nmsk(i+1,j).eq.0)) goto 2010
      seg(2,i,j-1)=.true.
      n = 3
      j = j-1
      goto iret
c
 3000 return
      end
c
c
c
      subroutine ngvala(nx,ny,x,y,z,xx,yy,zz,i,j,eps)
c     *****************************************************************
c     *  ngval - returns z-value at point (xx,yy) and its position
c     *          indices  
c     *
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real          z(nx,ny) : values of grid points
c     *   real             xx,yy : cordinate of input point
c     *   real                zz : value at point (xx,yy) 
c     *   integer            i,j : indices of point (xx,yy)
c     *   real               eps : limitter of grid area size
c     *
c     *****************************************************************
      dimension x(nx,ny),y(nx,ny),z(nx,ny)
      f(x1,y1,x2,y2,xx,yy)=(yy-y1)*(x2-x1)-(y2-y1)*(xx-x1)
      call ngij(nx,ny,x,y,1,1,nx,ny,xx,yy,i,j)
      if ((i.lt.1).or.(i.ge.nx)) goto 10
      if ((j.lt.1).or.(j.ge.ny)) goto 10
      x0=0.25*(x(i,j)+x(i+1,j)+x(i+1,j+1)+x(i,j+1))
      y0=0.25*(y(i,j)+y(i+1,j)+y(i+1,j+1)+y(i,j+1))
      z0=0.25*(z(i,j)+z(i+1,j)+z(i+1,j+1)+z(i,j+1))
      x1=x(i,j+1)
      y1=y(i,j+1)
      z1=z(i,j+1)
      x2=x(i,j)
      y2=y(i,j)
      z2=z(i,j)
      x3=x(i+1,j)
      y3=y(i+1,j)
      z3=z(i+1,j)
      x4=x(i+1,j+1)
      y4=y(i+1,j+1)
      z4=z(i+1,j+1)
c
      if ( (f(x0,y0,x2,y2,xx,yy).ge.0.0).and.
     &     (f(x0,y0,x3,y3,xx,yy).lt.0.0)) then
         x1=x2
         y1=y2
         z1=z2
         x2=x3
         y2=y3
         z2=z3
      elseif( (f(x0,y0,x3,y3,xx,yy).ge.0.0).and.
     &        (f(x0,y0,x4,y4,xx,yy).le.0.0)) then
         x1=x3
         y1=y3
         z1=z3
         x2=x4
         y2=y4
         z2=z4
      elseif( (f(x0,y0,x4,y4,xx,yy).ge.0.0).and.
     &        (f(x0,y0,x1,y1,xx,yy).le.0.0)) then
         x2=x1
         y2=y1
         z2=z1
         x1=x4
         y1=y4
         z1=z4
      endif
c
      dx1=x1-x0
      dy1=y1-y0
      dz1=z1-z0
      dx2=x2-x0
      dy2=y2-y0
      dz2=z2-z0
      dx = xx-x0
      dy = yy-y0
      denomi=dx1*dy2-dx2*dy1
      if (abs(denomi).lt.eps) then
         if (abs(dz1).ge.abs(dz2)) then
            if (abs(dx1).ge.abs(dy1)) then
               zz=z0+dz1/dx1*dx
            else
               zz=z0+dz1/dy1*dy
            endif
         else
            if (abs(dx2).ge.abs(dy2)) then
               zz=z0+dz2/dx2*dx
            else
               zz=z0+dz2/dy2*dy
            endif
         endif
      else   
         dzdx=(dz1*dy2-dz2*dy1)/denomi
         dzdy=(dz2*dx1-dz1*dx2)/denomi
         zz=z0+dzdx*dx+dzdy*dy
      endif
c
 10   return
      end
c
c
c
      subroutine ngvals(nx,ny,x,y,z,xx,yy,zz,i,j,eps,i1,j1,i2,j2,nmsk)
c     *****************************************************************
c     *  ngvals - returns z-value at point (xx,yy) and its position
c     *          indices  
c     *
c     *   integer          nx,ny : sizes of arrays
c     *   real x(nx,ny),y(nx,ny) : positions of grid points
c     *   real          z(nx,ny) : values of grid points
c     *   real             xx,yy : cordinate of input point
c     *   real                zz : value at point (xx,yy) 
c     *   integer            i,j : indices of point (xx,yy)
c     *   real               eps : limitter of grid area size
c     *   integer    i1,j1,i2,j2 : range of drowing by grid index
c     *   integer    nmsk(nx,ny) : mask array
c     *
c     *****************************************************************
      dimension x(nx,ny),y(nx,ny),z(nx,ny)
      integer nmsk(nx,ny)
      f(x1,y1,x2,y2,xx,yy)=(yy-y1)*(x2-x1)-(y2-y1)*(xx-x1)
      call ngij(nx,ny,x,y,i1,j1,i2,j2,xx,yy,i,j)
      if ((i.lt.i1).or.(i.ge.i2)) goto 10
      if ((j.lt.j1).or.(j.ge.j2)) goto 10
      if (((nmsk(i,j).ne.0).or.(nmsk(i+1,j).ne.0)).or.
     &     ((nmsk(i+1,j+1).ne.0).or.(nmsk(i,j+1).ne.0))) then
         i=-1
         j=-1
         zz=0
         goto 10
      endif
      x0=0.25*(x(i,j)+x(i+1,j)+x(i+1,j+1)+x(i,j+1))
      y0=0.25*(y(i,j)+y(i+1,j)+y(i+1,j+1)+y(i,j+1))
      z0=0.25*(z(i,j)+z(i+1,j)+z(i+1,j+1)+z(i,j+1))
      x1=x(i,j+1)
      y1=y(i,j+1)
      z1=z(i,j+1)
      x2=x(i,j)
      y2=y(i,j)
      z2=z(i,j)
      x3=x(i+1,j)
      y3=y(i+1,j)
      z3=z(i+1,j)
      x4=x(i+1,j+1)
      y4=y(i+1,j+1)
      z4=z(i+1,j+1)
c
      if ( (f(x0,y0,x2,y2,xx,yy).ge.0.0).and.
     &     (f(x0,y0,x3,y3,xx,yy).lt.0.0)) then
         x1=x2
         y1=y2
         z1=z2
         x2=x3
         y2=y3
         z2=z3
      elseif( (f(x0,y0,x3,y3,xx,yy).ge.0.0).and.
     &        (f(x0,y0,x4,y4,xx,yy).le.0.0)) then
         x1=x3
         y1=y3
         z1=z3
         x2=x4
         y2=y4
         z2=z4
      elseif( (f(x0,y0,x4,y4,xx,yy).ge.0.0).and.
     &        (f(x0,y0,x1,y1,xx,yy).le.0.0)) then
         x2=x1
         y2=y1
         z2=z1
         x1=x4
         y1=y4
         z1=z4
      endif
c
      dx1=x1-x0
      dy1=y1-y0
      dz1=z1-z0
      dx2=x2-x0
      dy2=y2-y0
      dz2=z2-z0
      dx = xx-x0
      dy = yy-y0
      denomi=dx1*dy2-dx2*dy1
      if (abs(denomi).lt.eps) then
         if (abs(dz1).ge.abs(dz2)) then
            if (abs(dx1).ge.abs(dy1)) then
               zz=z0+dz1/dx1*dx
            else
               zz=z0+dz1/dy1*dy
            endif
         else
            if (abs(dx2).ge.abs(dy2)) then
               zz=z0+dz2/dx2*dx
            else
               zz=z0+dz2/dy2*dy
            endif
         endif
      else   
         dzdx=(dz1*dy2-dz2*dy1)/denomi
         dzdy=(dz2*dx1-dz1*dx2)/denomi
         zz=z0+dzdx*dx+dzdy*dy
      endif
c
 10   return
      end
